## ----setup, cache = FALSE, message = FALSE------------------------------------
library(jrwb)
library(dplyr)

library(mkin)
library(nlme)
library(saemix)
saemix_options <- list(seed = 123456,
  nbiter.saemix = c(300, 100),
  print = FALSE, save = FALSE, save.graphs = FALSE, displayProgress = FALSE)

knitr::knit_hooks$set(fig.cap = function(before, options) {
  if (before) par(las = 1)
})
options(warn = 1) # We want warnings right away when they occur


## ----synthetic_common, echo = TRUE--------------------------------------------
sampling_times = c(0, 1, 3, 7, 14, 28, 60, 90, 120)

n_groups_sfo = 5
n_groups_biphasic = 8

trans_sd = 0.5
parent_0_mean = 100
parent_0_sd = 2

n_datasets = 100

err_twocomp = list(const = 1, prop = 0.07)


## ----synthetic_sfo, echo = TRUE, dependson = "synthetic_common", message = FALSE----
SFO <- mkinmod(parent = mkinsub("SFO"))

sfo_mean_dt50 <- c(15, 120, 500, 800)

ds_sfo <- function(geomean_dt50) {
  mean_parms_ds <- matrix(NA, nrow = n_datasets, ncol = 2,
    dimnames = list(1:n_datasets, c("parent_0", "k_parent")))
  ds <- lapply(1:n_datasets, function(i) {
    parent_0_ds <- rnorm(n_groups_sfo, parent_0_mean, parent_0_sd)
    k_parent_ds <- rlnorm(n_groups_sfo, log(log(2)/geomean_dt50), trans_sd)
    mean_parms_ds[i, "parent_0"] <<- mean(parent_0_ds)
    mean_parms_ds[i, "k_parent"] <<- geomean(k_parent_ds)

    groups_sfo <- lapply(1:n_groups_sfo, function(j) {
      group_prediction <- mkinpredict(SFO, c(k_parent = k_parent_ds[j]),
        c(parent = parent_0_ds[j]), sampling_times)
      group <- add_err(group_prediction, sdfunc = function(value) {
        sqrt(err_twocomp$const^2 + value^2 * err_twocomp$prop^2)
        }, n = 1)[[1]]
      return(group)
    })
    names(groups_sfo) <- paste("Group", 1:n_groups_sfo)
    return(groups_sfo)
  })
  list(ds = ds, mean_parms = mean_parms_ds)
}

set.seed(123456L)
sfo_data <- lapply(sfo_mean_dt50, ds_sfo)
names(sfo_data) <- paste(sfo_mean_dt50, "days")
ds_sfo_15 <- sfo_data[["15 days"]]$ds
ds_sfo_120 <- sfo_data[["120 days"]]$ds
ds_sfo_500 <- sfo_data[["500 days"]]$ds
ds_sfo_800 <- sfo_data[["800 days"]]$ds


## ----save_sfo_data------------------------------------------------------------
saveRDS(sfo_data, file = "sfo_data.rds")


## ----parms_sfo_synthetic, results = "asis"------------------------------------
parms_sfo_synthetic <- rbind(
  p_0 = c(rep(as.character(parent_0_mean), 4), parent_0_sd),
  half_life = c(paste(sfo_mean_dt50, "days"), ""),
  lambda = c(format(log(2)/sfo_mean_dt50, digits = 3, scientific = TRUE), ""),
  log_lambda = c(format(log(log(2)/sfo_mean_dt50), digits = 3, scientific = FALSE), trans_sd)
)

jrwb_table(parms_sfo_synthetic,
  caption = "Population parameter distributions used for the synthetic SFO datasets",
  label = "tab:parms_sfo_synthetic",
  group_names = c("Mode", "Standard deviation"),
  group_sizes = c(4, 1),
  header = c(paste("Variant", 1:4), "Variants 1-4"),
  factors = list(
    Parameter = c("$p_0$", "Half-life", "$\\lambda$", "$\\log( \\lambda$ days)"),
    Unit = c("[\\%]$^a$", "[days]", "[days$^{-1}$]", "[-]")),
  footers = "$^a$Percent of the intended initial residue"
)


## ----synthetic_biphasic, echo = TRUE, dependson = "synthetic_common"----------
n_datasets_biphasic = 100

DFOP_SFO <- mkinmod(
  parent = mkinsub("DFOP", "m1"),
  m1 = mkinsub("SFO"),
  unload = TRUE, overwrite = TRUE, quiet = TRUE)

mean_parms_biphasic_in <- matrix(NA, nrow = n_datasets_biphasic, ncol = 6)
colnames(mean_parms_biphasic_in) <- c("parent_0", "k_m1", "f_parent_to_m1",
  "k1", "k2", "g")

set.seed(123456L)
ds_biphasic <- lapply(1:n_datasets_biphasic, function(i) {

  parms_biphasic <- as.matrix(data.frame(
    parent_0 = rnorm(n_groups_biphasic, parent_0_mean, parent_0_sd),
    k1 = rlnorm(n_groups_biphasic, log(0.05), trans_sd),
    k2 = rlnorm(n_groups_biphasic, log(0.01), trans_sd),
    g = plogis(rnorm(n_groups_biphasic, qlogis(0.5), trans_sd)),
    f_parent_to_m1 = plogis(rnorm(n_groups_biphasic, qlogis(0.5), trans_sd)),
    k_m1 = rlnorm(n_groups_biphasic, log(0.002), trans_sd)))

  mean_parms_biphasic_in[i, "parent_0"] <<- mean(parms_biphasic[, "parent_0"])
  mean_parms_biphasic_in[i, "k_m1"] <<- geomean(parms_biphasic[, "k_m1"])
  mean_parms_biphasic_in[i, "f_parent_to_m1"] <<- plogis(mean(qlogis(parms_biphasic[, "f_parent_to_m1"])))
  mean_parms_biphasic_in[i, "k1"] <<- geomean(parms_biphasic[, "k1"])
  mean_parms_biphasic_in[i, "k2"] <<- geomean(parms_biphasic[, "k2"])
  mean_parms_biphasic_in[i, "g"] <<- plogis(mean(qlogis(parms_biphasic[, "g"])))

  ds_biphasic_mean <- lapply(1:n_groups_biphasic,
    function(i) {
      mkinpredict(DFOP_SFO, parms_biphasic[i, 2:6],
        c(parent = parms_biphasic[[i, 1]], m1 = 0), sampling_times)
    }
  )

  ds_biphasic_tmp <- lapply(ds_biphasic_mean, function(ds) {
    add_err(ds,
      sdfunc = function(value) sqrt(err_twocomp$const^2 + value^2 * err_twocomp$prop^2),
      n = 1, secondary = "m1")[[1]]
  })

  names(ds_biphasic_tmp) <- paste("Group", 1:n_groups_biphasic)
  return(ds_biphasic_tmp)

})


## ----parms_biphasic_synthetic, results = "asis"-------------------------------
parms_biphasic_synthetic <- rbind(
  p_0 = c(as.character(parent_0_mean), parent_0_sd),
  k1 = c(as.character(0.05), ""),
  log_k1 = c(format(log(0.05), digits = 3, scientific = FALSE), trans_sd),
  k2 = c(as.character(0.01), ""),
  log_k2 = c(format(log(0.01), digits = 3, scientific = FALSE), trans_sd),
  g = c(as.character(0.5), ""),
  g_qlogis = c(as.character(qlogis(0.5)), trans_sd),
  f_parent_to_m1 = c(as.character(0.5), ""),
  f_parent_to_m1_qlogis = c(as.character(qlogis(0.5)), trans_sd),
  k_m1 = c(as.character(0.002), ""),
  log_k_m1 = c(format(log(0.002), digits = 3, scientific = FALSE), trans_sd)
)

jrwb_table(parms_biphasic_synthetic,
  caption = "Population parameter distributions used for the synthetic DFOP-SFO datasets",
  label = "tab:parms_biphasic_synthetic",
  header = c("Mode", "Standard deviation"),
  factors = list(
    Parameter = c("$p_0$",
      "$\\lambda_1$", "$\\log( \\lambda_1$ days)",
      "$\\lambda_2$", "$\\log( \\lambda_2$ days)",
      "$\\gamma_1$", "logit $\\gamma_1$",
      "$\\gamma_2$", "logit $\\gamma_2$",
      "$\\lambda_3$", "$\\log( \\lambda_3$ days)"),
    Variable = c("\\texttt{parent\\_0}",
      "\\texttt{k1}", "",
      "\\texttt{k2}", "",
      "\\texttt{g}", "",
      "\\texttt{f\\_parent\\_to\\_m1}", "",
      "\\texttt{k\\_m1}", ""),
    Unit = c("[\\%]$^a$",
      "[days$^{-1}$]", "[-]",
      "[days$^{-1}$]", "[-]",
      "[-]", "[-]",
      "[-]", "[-]",
      "[days$^{-1}$]", "[-]")),
  footers = "$^a$Percent of the intended initial residue"
)


## ----save_biphasic_data-------------------------------------------------------
saveRDS(list(ds = ds_biphasic, mean_parms = mean_parms_biphasic_in), 
  file = "biphasic_data.rds")


## -----------------------------------------------------------------------------
variants = c("sep cst", "sep tc", "nlme cst", "nlme tc", "saem cst", "saem tc")


## ----fit_sfo_mkin_nlme--------------------------------------------------------
fit_sfo_mkin_nlme <- function(i, ds_sfo, verbose = FALSE) {

  message("Started ", i)

  res <- numeric(0)

  f_sfo <- mmkin("SFO", ds_sfo[[i]], quiet = TRUE, cores = 1)
  dt50_sfo <- sapply(f_sfo, function(x) endpoints(x)$distimes["parent", "DT50"])
  res["const"] <- pfm::geomean(dt50_sfo)

  f_sfo_tc <- mmkin("SFO", ds_sfo[[i]], error_model = "tc", quiet = TRUE, cores = 1)
  dt50_sfo_tc <- sapply(f_sfo_tc, function(x) endpoints(x)$distimes["parent", "DT50"])
  res["tc"] <- pfm::geomean(dt50_sfo_tc)

  # Make nlme warning about 'Singular precision matrix ...' an error to avoid infinite loop
  options(warn = 2)

  f_sfo_nlme <- try(nlme(f_sfo,
    control = list(maxIter = 50, msMaxIter = 50, msWarnNoConv = FALSE),
    verbose = verbose))
  if (!inherits(f_sfo_nlme, "try-error")) {
    res["nlme const"] <-
      as.numeric(log(2)/exp(summary(f_sfo_nlme)$coefficients$fixed["log_k_parent"]))
  } else {
    res["nlme const"] <- NA
  }

  f_sfo_nlme_tc <- try(nlme(f_sfo_tc,
    control = list(maxIter = 50, msMaxIter = 50, msWarnNoConv = FALSE),
    verbose = verbose))

  if (!inherits(f_sfo_nlme_tc, "try-error")) {
    res["nlme tc"] <-
      as.numeric(log(2)/exp(summary(f_sfo_nlme_tc)$coefficients$fixed["log_k_parent"]))
  } else {
    res["nlme tc"] <- NA
  }

  options(warn = 1) # Reset warnings to warnings

  message("Finished ", i)

  retval <- list(
    dt50 = res,
    sep_const = f_sfo,
    sep_tc = f_sfo_tc,
    nlme_const = f_sfo_nlme,
    nlme_tc = f_sfo_nlme_tc)

  return(retval)
}


## ----fit_sfo_saemix-----------------------------------------------------------
fit_sfo_saem <- function(res_mmkin) {
  try(
    saem(res_mmkin,
      transformations = "saemix",
      control = saemix_options, quiet = TRUE),
    silent = TRUE)
}


## ----fit_sfo_saemix_test------------------------------------------------------
fit_sfo_saem_test <- function(res_mmkin) {
  try(
    saem(res_mmkin,
      transformations = "saemix",
      test_log_parms = TRUE,
      control = saemix_options, quiet = TRUE),
    silent = TRUE)
}


## ----fit_sfo_mkin_nlme_15, dependson = c("synthetic_sfo", "fit_sfo_mkin_nlme"), message = FALSE, warning = FALSE----
res_sfo_mkin_nlme_15 <- parallel::mclapply(1:n_datasets,
  fit_sfo_mkin_nlme, ds_sfo_15,
  mc.preschedule = FALSE, mc.cores = 16)
dt50_sfo_geomean_15 <- as.data.frame(t(sapply(res_sfo_mkin_nlme_15, function(x) x[[1]])))
n_nlme_15 <- nrow(subset(dt50_sfo_geomean_15, !is.na(`nlme const`)))
n_nlme_tc_15 <- nrow(subset(dt50_sfo_geomean_15, !is.na(`nlme tc`)))
n_nlme_complete_15 <- nrow(subset(dt50_sfo_geomean_15, !is.na(`nlme const`) & !is.na(`nlme tc`)))


## ----fit_sfo_saemix_15, dependson = c("fit_sfo_saemix", "fit_sfo_mkin_nlme_15"), message = FALSE, warning = FALSE----
res_sfo_mmkin_15 <- lapply(res_sfo_mkin_nlme_15, function(x) x[[2]])
res_sfo_saemix_15 <- parallel::mclapply(res_sfo_mmkin_15, fit_sfo_saem,
  mc.cores = 1, mc.preschedule = TRUE)


## ----fit_sfo_saemix_tc_15, dependson = c("fit_sfo_saemix", "fit_sfo_mkin_nlme_15"), message = FALSE, warning = FALSE----
res_sfo_mmkin_tc_15 <- lapply(res_sfo_mkin_nlme_15, function(x) x[[3]])
res_sfo_saemix_tc_15 <-  parallel::mclapply(res_sfo_mmkin_tc_15, fit_sfo_saem,
  mc.cores = 1, mc.preschedule = TRUE)


## ----dt50_sfo_geomean_15, dependson = "fit_sfo_saemix_tc_15"------------------
dt50_sfo_geomean_15 <- cbind(data.frame(input = log(2)/sfo_data[["15 days"]]$mean_parms[,"k_parent"]),
  dt50_sfo_geomean_15)
dt50_sfo_geomean_15["saemix const"] <- sapply(res_sfo_saemix_15,
  function(x) log(2) / x$so@results@fixed.effects[2])
dt50_sfo_geomean_15["saemix tc"] = sapply(res_sfo_saemix_tc_15,
    function(x) log(2) / x$so@results@fixed.effects[2])


## ----dt50_15_plot_mmkin, fig.cap = "Separate fits to an example dataset with mean input half-life 15 days, constant variance error model", fig.height = 9, dependson = "fit_sfo_saemix_15"----
plot(res_sfo_mmkin_15[[1]], main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_15_plot_mmkin_tc, fig.cap = "Separate fits to an example dataset with mean input half-life 15 days, two-component error model", fig.height = 9, dependson = "fit_sfo_saemix_tc_15"----
plot(res_sfo_mmkin_tc_15[[1]], standardized = TRUE, main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_15_plot_mmkin_mixed, fig.cap = "Combined plot of separate fits to an example dataset with mean input half-life 15 days, constant variance error model", fig.height = 4, dependson = "fit_sfo_saemix_15"----
plot(mixed(res_sfo_mmkin_15[[1]]), main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_15_plot_mmkin_tc_mixed, fig.cap = "Combined plot of separate fits to an example dataset with mean input half-life 15 days, two-component error model", fig.height = 4, dependson = "fit_sfo_saemix_tc_15"----
plot(mixed(res_sfo_mmkin_tc_15[[1]]), standardized = TRUE, main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_15_plot_saem, fig.cap = "SFO fit with saemix to an example dataset with mean input half-life 15 days, constant variance", fig.height = 4, dependson = "fit_sfo_saemix_15"----
plot(res_sfo_saemix_15[[1]], xlab = "Time [days]")



## ----dt50_15_plot_saem_tc, fig.cap = "SFO fit with saemix to an example dataset with mean input half-life 15 days, two-component error model", fig.height = 4, dependson = "fit_sfo_saemix_tc_15"----
plot(res_sfo_saemix_tc_15[[1]], xlab = "Time [days]")



## ----plot_sfo_15, fig.cap = paste("Boxplots of half-lives derived from", n_datasets, "datasets for five groups, input 15 days"), fig.height = 4.5, dependson = "dt50_sfo_geomean_15", message = FALSE----
boxplot_sfo <- function(dt50_wide, dt50, ylim_add = 5, ylim_max = NULL, ...)
{
  colnames(dt50_wide) <- c("Input", "sep cst", "sep tc",
    "nlme cst", "nlme tc", "saem cst", "saem tc")

  if (is.null(ylim_max)) {
    ylim = range(dt50_wide, na.rm = TRUE) + c(0, ylim_add)
  } else {
    ylim = c(min(dt50_wide, na.rm = TRUE), ylim_max)
  }

  boxplot(dt50_wide, log = "y",
    at = c(1, 3, 4, 6, 7, 9, 10),
    ylim = ylim,
    col = c("grey", rep(c("white", "darkgrey"), 3)),
    ylab = "Population half-life [days]",
    xaxt = "n"
  )
  axis(1, at = c(1, 3.5, 6.5, 9.5), tick = FALSE,
    labels = c("Input samples", "Separate fits", "nlme", "saemix"))

  abline(h = dt50, lty = 2, lwd = 2, col = "black")
  quartiles_k <- qlnorm(c(0.25, 0.75),
    log(log(2)/dt50),
    trans_sd/sqrt(n_groups_sfo))
  abline(h = log(2)/quartiles_k, lwd = 2, lty = 3, col = "black")
  legend("topleft", bty = "n", lty = 2:3, lwd = 2,
    title = "Distribution of input sample means",
    legend = c("Mode", "Quartiles"))
  legend("topright", bty = "n", fill = c("white", "darkgrey"),
    title = "Error model",
    legend = c("Constant variance", "Two-component error"))
}
boxplot_sfo(dt50_sfo_geomean_15, 15, 10)


## ----stats_sfo, results = "asis", dependson = "synthetic_sfo", message = FALSE----
stats_sfo <- function(dt50, dt50_sfo_geomean, res_sfo_mkin_nlme, res_sfo_saemix, res_sfo_saemix_tc) {

  stats_n <- sapply(dt50_sfo_geomean, function(col) sum(!is.na(col)))
  stats_n[1] <- NA # This is the input, and not a real result
  stats_n[2:3] <- 5 * stats_n[2:3] # we have five individual results for each result
  quantiles <- sapply(dt50_sfo_geomean, quantile, c(0, 0.25, 0.5, 0.75, 1), na.rm = TRUE)

  aic_count <- rep(0, 6)
  names(aic_count) <- variants

  for (i in 1:n_datasets) {
    for (j in 1:5) {
      aic_diff_mkin <- try(AIC(res_sfo_mkin_nlme[[i]][["sep_const"]][[j]]) -
        AIC(res_sfo_mkin_nlme[[i]][["sep_tc"]][[j]]), silent = TRUE)
      if (!inherits(aic_diff_mkin, "try-error")) {
        if (aic_diff_mkin < 0) aic_count["sep cst"] <- aic_count["sep cst"] + 1
        else aic_count["sep tc"] <- aic_count["sep tc"] + 1
      }
    }
    aic_diff_nlme <- try(AIC(res_sfo_mkin_nlme[[i]][["nlme_const"]]) -
      AIC(res_sfo_mkin_nlme[[i]][["nlme_tc"]]), silent = TRUE)
    if (!inherits(aic_diff_nlme, "try-error")) {
      if (aic_diff_nlme < 0) aic_count["nlme cst"] <- aic_count["nlme cst"] + 1
      else aic_count["nlme tc"] <- aic_count["nlme tc"] + 1
    }
    aic_diff_saem <- try(res_sfo_saemix[[i]]$so@results@aic.is -
      res_sfo_saemix_tc[[i]]$so@results@aic.is, silent = TRUE)
    if (!inherits(aic_diff_saem, "try-error")) {
      if (aic_diff_saem < 0) aic_count["saem cst"] <- aic_count["saem cst"] + 1
      else aic_count["saem tc"] <- aic_count["saem tc"] + 1
    }
  }
  theoretical_k <- qlnorm(c(0.75, 0.5, 0.25),
    log(log(2)/dt50),
    trans_sd/sqrt(n_groups_sfo))
  theoretical_dt50 <- c(rep(NA, 3),
    format(log(2)/theoretical_k, digits = 4), NA)

  jrwb_table(cbind(theoretical_dt50, rbind(stats_n, c(NA, aic_count), format(quantiles, digits = 4))),
    size = "small",
    caption = paste("Statistics for half-lives derived from", n_datasets, "datasets with five groups, input half-life ", dt50, " days"),
    label = paste0("tab:dt50_", dt50, "_stats"),
    factors = list(c("Number of results", "Fits with lower AIC",
        "Minimum", "25th percentile", "Median", "75th percentile", "Maximum")),
    midrules = c(1, 2),
    header = c("Distribution", "Sample means", "const", "tc", "const", "tc", "const", "tc"),
    group_names = c("Input parameters", "Separate fits", "nlme", "saemix"),
    group_sizes = c(2, 2, 2, 2)
  )

}


## ----dt50_15_stats, results = "asis", dependson = c("stats_sfo", "plot_sfo_15")----
stats_sfo(15, dt50_sfo_geomean_15, res_sfo_mkin_nlme_15, res_sfo_saemix_15, res_sfo_saemix_tc_15)


## ----fit_sfo_mkin_nlme_120, dependson = c("synthetic_sfo", "fit_sfo_mkin_nlme"), message = FALSE, warning = FALSE----
res_sfo_mkin_nlme_120 <- parallel::mclapply(1:n_datasets,
  fit_sfo_mkin_nlme, ds_sfo_120,
  mc.preschedule = FALSE, mc.cores = 16)
dt50_sfo_geomean_120 <- as.data.frame(t(sapply(res_sfo_mkin_nlme_120, function(x) x[[1]])))
n_nlme_120 <- nrow(subset(dt50_sfo_geomean_120, !is.na(`nlme const`)))
n_nlme_tc_120 <- nrow(subset(dt50_sfo_geomean_120, !is.na(`nlme tc`)))
n_nlme_complete_120 <- nrow(subset(dt50_sfo_geomean_120, !is.na(`nlme const`) & !is.na(`nlme tc`)))


## ----fit_sfo_saemix_120, dependson = c("fit_sfo_saemix", "fit_sfo_mkin_nlme_120"), message = FALSE, warning = FALSE----
res_sfo_mmkin_120 <- lapply(res_sfo_mkin_nlme_120, function(x) x[[2]])
res_sfo_saemix_120 <- parallel::mclapply(res_sfo_mmkin_120, fit_sfo_saem,
  mc.cores = 1, mc.preschedule = TRUE)


## ----fit_sfo_saemix_tc_120, dependson = c("fit_sfo_saemix", "fit_sfo_mkin_nlme_120"), message = FALSE, warning = FALSE----
res_sfo_mmkin_tc_120 <- lapply(res_sfo_mkin_nlme_120, function(x) x[[3]])
res_sfo_saemix_tc_120 <-  parallel::mclapply(res_sfo_mmkin_tc_120, fit_sfo_saem,
   mc.cores = 1, mc.preschedule = TRUE)


## ----dt50_sfo_geomean_120, dependson = "fit_sfo_saemix_tc_120"----------------
dt50_sfo_geomean_120 <- cbind(data.frame(input = log(2)/sfo_data[["120 days"]]$mean_parms[,"k_parent"]),
  dt50_sfo_geomean_120)
dt50_sfo_geomean_120["saemix const"] <- sapply(res_sfo_saemix_120,
  function(x) log(2) / x$so@results@fixed.effects[2])
dt50_sfo_geomean_120["saemix_tc"] = sapply(res_sfo_saemix_tc_120,
    function(x) log(2) / x$so@results@fixed.effects[2])


## ----dt50_120_plot_mmkin, fig.cap = "Separate fits to an example dataset with mean input half-life 120 days, constant variance error model", fig.height = 9, dependson = "fit_sfo_mkin_nlme_120"----
plot(res_sfo_mmkin_120[[1]], main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_120_plot_mmkin_tc, fig.cap = "Separate fits to an example dataset with mean input half-life 120 days, two-component error model", fig.height = 9, dependson = "fit_sfo_mkin_nlme_120"----
plot(res_sfo_mmkin_tc_120[[1]], standardized = TRUE, main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_120_plot_mmkin_mixed, fig.cap = "Combined plot of separate fits to an example dataset with mean input half-life 120 days, constant variance error model", fig.height = 4, dependson = "fit_sfo_mkin_nlme_120"----
plot(mixed(res_sfo_mmkin_120[[1]]), main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_120_plot_mmkin_tc_mixed, fig.cap = "Combined plot of separate fits to an example dataset with mean input half-life 120 days, two-component error model", fig.height = 4, dependson = "fit_sfo_mkin_nlme_120"----
plot(mixed(res_sfo_mmkin_tc_120[[1]]), standardized = TRUE, main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_120_plot_saem, fig.cap = "SFO fit with saemix to an example dataset with mean input half-life 120 days, constant variance", fig.height = 4, dependson = "fit_sfo_saemix_120"----
plot(res_sfo_saemix_120[[1]], xlab = "Time [days]")



## ----dt50_120_plot_saem_tc, fig.cap = "SFO fit with saemix to an example dataset with mean input half-life 120 days, two-component error model", fig.height = 4, dependson = "fit_sfo_saemix_120"----
plot(res_sfo_saemix_tc_120[[1]], xlab = "Time [days]")



## ----plot_sfo_120, fig.cap = paste("Boxplots of half-lives derived from", n_datasets, "datasets for five groups, input 120 days"), fig.height = 4.5, dependson = c("plot_sfo_15", "dt50_sfo_geomean_120")----
boxplot_sfo(dt50_sfo_geomean_120, 120, 100)


## ----dt50_120_stats, results = "asis", dependson = c("stats_sfo", "plot_sfo_120")----
stats_sfo(120, dt50_sfo_geomean_120, res_sfo_mkin_nlme_120, res_sfo_saemix_120, res_sfo_saemix_tc_120)


## ----fit_sfo_mkin_nlme_500, dependson = c("synthetic_sfo", "fit_sfo_mkin_nlme"), message = FALSE, warning = FALSE----
res_sfo_mkin_nlme_500 <- parallel::mclapply(1:n_datasets,
  fit_sfo_mkin_nlme, ds_sfo_500,
  mc.preschedule = FALSE, mc.cores = 16)
dt50_sfo_geomean_500 <- as.data.frame(t(sapply(res_sfo_mkin_nlme_500, function(x) x[[1]])))
n_nlme_500 <- nrow(subset(dt50_sfo_geomean_500, !is.na(`nlme const`)))
n_nlme_tc_500 <- nrow(subset(dt50_sfo_geomean_500, !is.na(`nlme tc`)))
n_nlme_complete_500 <- nrow(subset(dt50_sfo_geomean_500, !is.na(`nlme const`) & !is.na(`nlme tc`)))


## ----fit_sfo_saemix_500, dependson = c("fit_sfo_saemix", "fit_sfo_mkin_nlme_500"), message = FALSE, warning = FALSE----
res_sfo_mmkin_500 <- lapply(res_sfo_mkin_nlme_500, function(x) x[[2]])
res_sfo_saemix_500 <- parallel::mclapply(res_sfo_mmkin_500, fit_sfo_saem,
  mc.cores = 1, mc.preschedule = TRUE)


## ----fit_sfo_saemix_tc_500, dependson = c("fit_sfo_saemix", "fit_sfo_mkin_nlme_500"), message = FALSE, warning = FALSE----
res_sfo_mmkin_tc_500 <- lapply(res_sfo_mkin_nlme_500, function(x) x[[3]])
res_sfo_saemix_tc_500 <-  parallel::mclapply(res_sfo_mmkin_tc_500, fit_sfo_saem,
  mc.cores = 1, mc.preschedule = TRUE)


## ----dt50_sfo_geomean_500, dependson = "fit_sfo_saemix_tc_15"-----------------
dt50_sfo_geomean_500 <- cbind(data.frame(input = log(2)/sfo_data[["500 days"]]$mean_parms[,"k_parent"]),
  dt50_sfo_geomean_500)
dt50_sfo_geomean_500["saemix const"] <- sapply(res_sfo_saemix_500,
  function(x) if(inherits(x, "try-error")) NA else log(2) / x$so@results@fixed.effects[2])
dt50_sfo_geomean_500["saemix tc"] = sapply(res_sfo_saemix_tc_500,
    function(x) if(inherits(x, "try-error")) NA else log(2) / x$so@results@fixed.effects[2])


## ----dt50_500_plot_mmkin, fig.cap = "Separate fits to an example dataset with mean input half-life 500 days, constant variance error model", fig.height = 9, dependson = "fit_sfo_mkin_nlme_500"----
plot(res_sfo_mmkin_500[[1]], main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_500_plot_mmkin_tc, fig.cap = "Separate fits to an example dataset with mean input half-life 500 days, two-component error model", fig.height = 9, dependson = "fit_sfo_mkin_nlme_500"----
plot(res_sfo_mmkin_tc_500[[1]], standardized = TRUE, main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_500_plot_mmkin_mixed, fig.cap = "Combined plot of separate fits to an example dataset with mean input half-life 500 days, constant variance error model", fig.height = 4, dependson = "fit_sfo_mkin_nlme_500"----
plot(mixed(res_sfo_mmkin_500[[1]]), main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_500_plot_mmkin_tc_mixed, fig.cap = "Combined plot of separate fits to an example dataset with mean input half-life 500 days, two-component error model", fig.height = 4, dependson = "fit_sfo_mkin_nlme_500"----
plot(mixed(res_sfo_mmkin_tc_500[[1]]), standardized = TRUE, main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_500_plot_saem, fig.cap = "SFO fit with saemix to an example dataset with mean input half-life 500 days, constant variance", fig.height = 4, dependson = "fit_sfo_saemix_500"----
plot(res_sfo_saemix_500[[1]], xlab = "Time [days]")



## ----dt50_500_plot_saem_tc, fig.cap = "SFO fit with saemix to an example dataset with mean input half-life 500 days, two-component error model", fig.height = 4, dependson = "fit_sfo_saemix_tc_500"----
plot(res_sfo_saemix_tc_500[[1]], xlab = "Time [days]")



## ----plot_sfo_500, fig.cap = paste("Boxplots of half-lives derived from", n_datasets, "datasets for five groups, input 500 days"), fig.height = 4.5, dependson = c("plot_sfo_15", "fit_sfo_saemix_tc_500")----
boxplot_sfo(dt50_sfo_geomean_500, 500, ylim_max = 2e4)


## ----dt50_500_stats, results = "asis", dependson = c("stats_sfo", "plot_sfo_500")----
stats_sfo(500, dt50_sfo_geomean_500, res_sfo_mkin_nlme_500, res_sfo_saemix_500, res_sfo_saemix_tc_500)


## ----fit_sfo_mkin_nlme_800, dependson = c("synthetic_sfo", "fit_sfo_mkin_nlme"), message = FALSE, warning = FALSE----
res_sfo_mkin_nlme_800 <- parallel::mclapply(1:n_datasets,
  fit_sfo_mkin_nlme, ds_sfo_800,
  mc.preschedule = FALSE, mc.cores = 8)
dt50_sfo_geomean_800 <- as.data.frame(t(sapply(res_sfo_mkin_nlme_800, function(x) x[[1]])))
n_nlme_800 <- nrow(subset(dt50_sfo_geomean_800, !is.na(`nlme const`)))
n_nlme_tc_800 <- nrow(subset(dt50_sfo_geomean_800, !is.na(`nlme tc`)))
n_nlme_complete_800 <- nrow(subset(dt50_sfo_geomean_800, !is.na(`nlme const`) & !is.na(`nlme tc`)))


## ----fit_sfo_saemix_800, dependson = c("fit_sfo_saemix_test", "fit_sfo_mkin_nlme_800"), message = FALSE, warning = FALSE, results = "hide"----
res_sfo_mmkin_800 <- lapply(res_sfo_mkin_nlme_800, function(x) x[[2]])
res_sfo_saemix_800 <- parallel::mclapply(res_sfo_mmkin_800, fit_sfo_saem_test,
  mc.cores = 1, mc.preschedule = TRUE)


## ----fit_sfo_saemix_tc_800, dependson = c("fit_sfo_saemix_test", "fit_sfo_mkin_nlme_800"), message = FALSE, warning = FALSE, results = "hide"----
res_sfo_mmkin_tc_800 <- lapply(res_sfo_mkin_nlme_800, function(x) x[[3]])
res_sfo_saemix_tc_800 <-  parallel::mclapply(res_sfo_mmkin_tc_800, fit_sfo_saem_test,
   mc.cores = 1, mc.preschedule = TRUE)


## ----dt50_sfo_geomean_800, dependson = c("fit_sfo_saemix_800", "fit_sfo_saemix_tc_800")----
dt50_sfo_geomean_800 <- cbind(data.frame(input = log(2)/sfo_data[["800 days"]]$mean_parms[,"k_parent"]),
  dt50_sfo_geomean_800)
dt50_sfo_geomean_800["saemix const"] <- sapply(res_sfo_saemix_800,
  function(x) if(inherits(x, "try-error")) NA else log(2) / x$so@results@fixed.effects[2])
dt50_sfo_geomean_800["saemix tc"] = sapply(res_sfo_saemix_tc_800,
    function(x) if(inherits(x, "try-error")) NA else log(2) / x$so@results@fixed.effects[2])


## ----dt50_800_plot_mmkin, fig.cap = "Separate fits to an example dataset with mean input half-life 800 days, constant variance error model", fig.height = 9, dependson = "fit_sfo_mkin_nlme_800"----
plot(res_sfo_mmkin_800[[7]], main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_800_plot_mmkin_tc, fig.cap = "Separate fits to an example dataset with mean input half-life 800 days, two-component error model", fig.height = 9, dependson = "fit_sfo_mkin_nlme_800"----
plot(res_sfo_mmkin_tc_800[[7]], standardized = TRUE, main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_800_plot_mmkin_mixed, fig.cap = "Combined plot of separate fits to an example dataset with mean input half-life 800 days, constant variance error model", fig.height = 4, dependson = "fit_sfo_mkin_nlme_800"----
plot(mixed(res_sfo_mmkin_800[[7]]), main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_800_plot_mmkin_tc_mixed, fig.cap = "Combined plot of separate fits to an example dataset with mean input half-life 800 days, two-component error model", fig.height = 4, dependson = "fit_sfo_mkin_nlme_800"----
plot(mixed(res_sfo_mmkin_tc_800[[7]]), standardized = TRUE, main = "", legend = FALSE, xlab = "Time [days]")



## ----dt50_800_plot_saem, fig.cap = "SFO fit with saemix to an example dataset with mean input half-life 800 days, constant variance", fig.height = 4, dependson = "fit_sfo_saemix_800"----
plot(res_sfo_saemix_800[[7]], xlab = "Time [days]")



## ----dt50_800_plot_saem_tc, fig.cap = "SFO fit with saemix to an example dataset with mean input half-life 800 days, two-component error model", fig.height = 4, dependson = "fit_sfo_saemix_800"----
plot(res_sfo_saemix_tc_800[[7]], xlab = "Time [days]")



## ----dt50_800_plot_mmkin_saem, fig.cap = "SFO fits with mkin and saemix to an example dataset with mean input half-life 800 days, constant variance", fig.height = 4, dependson = "fit_sfo_mkin_saemix_800", message = FALSE----
# Plotting code adapted from plot.mixed.mmkin() in the mkin package
x <- mixed(res_sfo_mmkin_800[[7]])
y <- res_sfo_saemix_800[[7]]
xlab <- "Time [days]"
# Only guaranteed to work if x and y share the same mkinmod
i = 1:ncol(x$mmkin)
obs_vars = names(x$mkinmod$map)
xlab = "Time"
xlim = range(x$data$time)
test_log_parms = FALSE
conf.level = 0.6
ymax = "auto"
maxabs = "auto"
ncol.legend = ifelse(length(i) <= 3, length(i) + 1, ifelse(length(i) <= 8, 3, 4))
nrow.legend = ceiling((length(i) + 1) / ncol.legend)
rel.height.legend = 0.02 + 0.07 * nrow.legend
rel.height.bottom = 1.1
pch_ds = 1:length(i)
col_ds = pch_ds + 1
lty_ds = col_ds
frame = TRUE

# Prepare parameters and data
fit_1 <- x$mmkin[[1]]
ds_names <- colnames(x$mmkin)

backtransform_x <- TRUE
backtransform_y <- TRUE
if (!identical(class(x), "mixed.mmkin")) stop("x must be mixed.mmkin")

degparms_pop <- mean_degparms(x$mmkin, test_log_parms = test_log_parms, conf.level = conf.level)

degparms_tmp <- parms(x$mmkin, transformed = TRUE)
degparms_i <- as.data.frame(t(degparms_tmp[setdiff(rownames(degparms_tmp), names(fit_1$errparms)), ]))

if (inherits(y, "nlme.mmkin")) {
  degparms_i_y <- coefficients(y)
  degparms_pop_y <- nlme::fixef(y)
}

if (inherits(y, "saem.mmkin")) {
  if (y$transformations == "saemix") backtransform_y = FALSE
  degparms_i_y <- saemix::psi(y$so)
  rownames(degparms_i_y) <- ds_names
  degparms_i_names_y <- setdiff(y$so@results@name.fixed, names(fit_1$errparms))
  colnames(degparms_i_y) <- degparms_i_names_y
  degparms_pop_y <- y$so@results@fixed.effects
  names(degparms_pop_y) <- degparms_i_names_y
}

if (inherits(y, "nlmixr.mmkin")) {
  eta_i <- random.effects(y$nm)[-1]
  names(eta_i) <- gsub("^eta.", "", names(eta_i))
  degparms_i_y <- eta_i
  degparms_pop_y <- y$nm$theta
  for (parm_name in names(degparms_i_y)) {
    degparms_i_y[parm_name] <- eta_i[parm_name] + degparms_pop_y[parm_name]
  }
}

degparms_fixed <- fit_1$fixed$value
names(degparms_fixed) <- rownames(fit_1$fixed)

# x
degparms_all <- cbind(as.matrix(degparms_i),
  matrix(rep(degparms_fixed, nrow(degparms_i)),
    ncol = length(degparms_fixed),
    nrow = nrow(degparms_i), byrow = TRUE))
degparms_all_names <- c(names(degparms_i), names(degparms_fixed))
colnames(degparms_all) <- degparms_all_names

degparms_all_pop <- c(degparms_pop, degparms_fixed)

odeini_names <- grep("_0$", degparms_all_names, value = TRUE)
odeparms_names <- setdiff(degparms_all_names, odeini_names)

observed <- x$data[c("ds", "name", "time", "value")]

solution_type = fit_1$solution_type

outtimes <- sort(unique(c(x$data$time,
  seq(xlim[1], xlim[2], length.out = 50))))

pred_ds <- purrr::map_dfr(i, function(ds_i)   {
  odeparms_trans <- degparms_all[ds_i, odeparms_names]
  names(odeparms_trans) <- odeparms_names # needed if only one odeparm
  if (backtransform_x) {
    odeparms <- backtransform_odeparms(odeparms_trans,
      x$mkinmod,
      transform_rates = fit_1$transform_rates,
      transform_fractions = fit_1$transform_fractions)
  } else {
    odeparms <- odeparms_trans
  }

  odeini <- degparms_all[ds_i, odeini_names]
  names(odeini) <- gsub("_0", "", odeini_names)

  out <- mkinpredict(x$mkinmod, odeparms, odeini,
    outtimes, solution_type = solution_type,
    atol = fit_1$atol, rtol = fit_1$rtol)
  return(cbind(as.data.frame(out), ds = ds_names[ds_i]))
})

odeparms_pop_trans <- degparms_all_pop[odeparms_names]

if (backtransform_x) {
  odeparms_pop <- backtransform_odeparms(odeparms_pop_trans,
    x$mkinmod,
    transform_rates = fit_1$transform_rates,
    transform_fractions = fit_1$transform_fractions)
} else {
  odeparms_pop <- odeparms_pop_trans
}

odeini_pop <- degparms_all_pop[odeini_names]
names(odeini_pop) <- gsub("_0", "", odeini_names)

pred_pop <- as.data.frame(
  mkinpredict(x$mkinmod, odeparms_pop, odeini_pop,
    outtimes, solution_type = solution_type,
    atol = fit_1$atol, rtol = fit_1$rtol))

# y
degparms_all_y <- cbind(as.matrix(degparms_i_y),
  matrix(rep(degparms_fixed, nrow(degparms_i_y)),
    ncol = length(degparms_fixed),
    nrow = nrow(degparms_i_y), byrow = TRUE))
degparms_all_names_y <- c(names(degparms_i_y), names(degparms_fixed))
colnames(degparms_all_y) <- degparms_all_names_y

degparms_all_pop_y <- c(degparms_pop_y, degparms_fixed)
odeparms_names_y <- setdiff(degparms_all_names_y, odeini_names)

pred_ds_y <- purrr::map_dfr(i, function(ds_i)   {
  odeparms_trans <- degparms_all_y[ds_i, odeparms_names_y]
  names(odeparms_trans) <- odeparms_names_y # needed if only one odeparm
  if (backtransform_y) {
    odeparms <- backtransform_odeparms(odeparms_trans,
      y$mkinmod,
      transform_rates = fit_1$transform_rates,
      transform_fractions = fit_1$transform_fractions)
  } else {
    odeparms <- odeparms_trans
  }

  odeini <- degparms_all_y[ds_i, odeini_names]
  names(odeini) <- gsub("_0", "", odeini_names)

  out <- mkinpredict(y$mkinmod, odeparms, odeini,
    outtimes, solution_type = solution_type,
    atol = fit_1$atol, rtol = fit_1$rtol)
  return(cbind(as.data.frame(out), ds = ds_names[ds_i]))
})

odeparms_pop_trans_y <- degparms_all_pop_y[odeparms_names_y]

if (backtransform_y) {
  odeparms_pop_y <- backtransform_odeparms(odeparms_pop_trans_y,
    y$mkinmod,
    transform_rates = fit_1$transform_rates,
    transform_fractions = fit_1$transform_fractions)
} else {
  odeparms_pop_y <- odeparms_pop_trans_y
}

odeini_pop_y <- degparms_all_pop_y[odeini_names]
names(odeini_pop_y) <- gsub("_0", "", odeini_names)

pred_pop_y <- as.data.frame(
  mkinpredict(y$mkinmod, odeparms_pop_y, odeini_pop_y,
    outtimes, solution_type = solution_type,
    atol = fit_1$atol, rtol = fit_1$rtol))

n_plot_rows = length(obs_vars)
n_plots = n_plot_rows * 2

# Set relative plot heights, so the first plot row is the norm
rel.heights <- if (n_plot_rows > 1) {
  c(rel.height.legend, c(rep(1, n_plot_rows - 1), rel.height.bottom))
} else {
  c(rel.height.legend, 1)
}

layout_matrix = matrix(c(1, 1, 2:(n_plots + 1)),
  n_plot_rows + 1, 2, byrow = TRUE)
layout(layout_matrix, heights = rel.heights)

par(mar = c(0.1, 2.1, 0.1, 2.1))

# Empty plot with legend
n_pop <- 1
lty_pop <- 1

plot(0, type = "n", axes = FALSE, ann = FALSE)
legend("center", bty = "n", ncol = ncol.legend,
  legend = c("Population", ds_names[i]),
  lty = c(lty_pop, lty_ds),
  lwd = c(rep(2, n_pop), rep(1, length(i))),
  col = c(rep(1, n_pop), col_ds),
  pch = c(rep(NA, n_pop), pch_ds))

# Loop plot rows
for (plot_row in 1:n_plot_rows) {

  obs_var <- obs_vars[plot_row]
  observed_row <- subset(observed, name == obs_var)

  ylim_row = c(50,
      max(c(observed_row$value, pred_ds[[obs_var]]), na.rm = TRUE))

  # Margins for bottom row of plots when we have more than one row
  # This is the only row that needs to show the x axis legend
  if (plot_row == n_plot_rows) {
    par(mar = c(5.1, 4.1, 1.1, 2.1))
  } else {
    par(mar = c(3.0, 4.1, 1.1, 2.1))
  }

  plot(pred_pop$time, pred_pop[[obs_var]],
    type = "l", lwd = 2, lty = lty_pop,
    xlim = xlim, ylim = ylim_row,
    xlab = xlab, ylab = paste("Residues", obs_var), frame = frame)
  mtext("Separate fits", line = 0.5)

  for (ds_i in seq_along(i)) {
    points(subset(observed_row, ds == ds_names[ds_i], c("time", "value")),
      col = col_ds[ds_i], pch = pch_ds[ds_i])
    lines(subset(pred_ds, ds == ds_names[ds_i], c("time", obs_var)),
      col = col_ds[ds_i], lty = lty_ds[ds_i])
  }

  plot(pred_pop_y$time, pred_pop_y[[obs_var]],
    type = "l", lwd = 2, lty = lty_pop,
    xlim = xlim, ylim = ylim_row,
    xlab = xlab, ylab = paste("Residues", obs_var), frame = frame)
  mtext("Nonlinear mixed-effects model fit", line = 0.5)

  for (ds_i in seq_along(i)) {
    points(subset(observed_row, ds == ds_names[ds_i], c("time", "value")),
      col = col_ds[ds_i], pch = pch_ds[ds_i])
    lines(subset(pred_ds_y, ds == ds_names[ds_i], c("time", obs_var)),
      col = col_ds[ds_i], lty = lty_ds[ds_i])
  }
}


## ----plot_sfo_800, fig.cap = paste("Boxplots for the half-lives derived from", n_datasets, "datasets for five groups, half-life 800 days"), fig.height = 4.5, dependson = c("plot_sfo_15", "dt50_sfo_geomean_800"), message = FALSE----
boxplot_sfo(dt50_sfo_geomean_800, 800, ylim_max = 2e4)


## ----dt50_800_stats, results = "asis", dependson = c("stats_sfo", "plot_sfo_800", "dt50_sfo_geomean_800"), message = FALSE----
stats_sfo(800, dt50_sfo_geomean_800, res_sfo_mkin_nlme_800, res_sfo_saemix_800, res_sfo_saemix_tc_800)


## ----fit_biphasic_mkin_const, dependson = "synthetic_biphasic"----------------
SFO_SFO <- mkinmod(
  parent = mkinsub("SFO", "m1"),
  m1 = mkinsub("SFO"),
  unload = TRUE, overwrite = TRUE, quiet = TRUE)

fits_biphasic_mkin_const <- mclapply(ds_biphasic,
  function(ds) mmkin(list("SFO-SFO" = SFO_SFO, "DFOP-SFO" = DFOP_SFO),
    ds, cores = 1, quiet = TRUE, error_model = "const"),
  mc.preschedule = FALSE, mc.cores = 12)


## ----fit_biphasic_mkin_tc, dependson = "synthetic_biphasic"-------------------
fits_biphasic_mkin_tc <- mclapply(ds_biphasic,
  function(ds) mmkin(list("SFO-SFO" = SFO_SFO, "DFOP-SFO" = DFOP_SFO),
    ds, cores = 1, quiet = TRUE, error_model = "tc"),
  mc.preschedule = FALSE, mc.cores = 10)


## ----fit_biphasic_nlme_const, dependson = "fit_biphasic_mkin_const", eval = FALSE----
## fits_biphasic_nlme_const_sfo_sfo <- mclapply(fits_biphasic_mkin_const[1:10],
##   function(f) nlme(f[1, ], control = list(msWarnNoConv = FALSE)),
##   mc.preschedule = FALSE, mc.cores = 5)
## # 2/16 function calls resulted in an error
## fits_biphasic_nlme_const_dfop_sfo <- mclapply(fits_biphasic_mkin_const,
##   function(f) nlme(f[2, ], control = list(msWarnNoConv = FALSE)),
##   mc.preschedule = FALSE, mc.cores = 8)
## # 7/16 function calls resulted in an error


## ----fit_biphasic_saem_const_sfo_sfo, dependson = "fit_biphasic_mkin_const", results = "hide", message = FALSE----
fits_biphasic_saem_const_sfo_sfo <- mclapply(1:n_datasets_biphasic,
  function(i) {
    message("saem SFO-SFO fit const ", i)
    try(
      saem(fits_biphasic_mkin_const[[i]][1, ], transformations = "saemix",
        fail_with_errors = TRUE, test_log_parms = TRUE, quiet = TRUE),
      silent = TRUE)
  }, mc.cores = 1)


## ----fit_biphasic_saem_const_dfop_sfo, dependson = "fit_biphasic_mkin_const", results = "hide", message = FALSE----
fits_biphasic_saem_const_dfop_sfo <- mclapply(1:n_datasets_biphasic,
  function(i) {
    message("saem DFOP-SFO fit const ", i)
    try(
      saem(fits_biphasic_mkin_const[[i]][2, ], transformations = "saemix",
        fail_with_errors = TRUE, test_log_parms = TRUE, quiet = TRUE),
      silent = TRUE)
  }, mc.cores = 1)


## ----fit_biphasic_saem_tc_sfo_sfo, dependson = "fit_biphasic_mkin_tc", results = "hide", message = FALSE----
fits_biphasic_saem_tc_sfo_sfo <- mclapply(1:n_datasets_biphasic,
  function(i) {
    message("saem SFO-SFO fit tc ", i)
    try(
      saem(fits_biphasic_mkin_tc[[i]][1, ], transformations = "saemix",
        fail_with_errors = TRUE, test_log_parms = TRUE, quiet = TRUE),
      silent = TRUE)
  }, mc.cores = 1)


## ----fit_biphasic_saem_tc_dfop_sfo, dependson = "fit_biphasic_mkin_tc", results = "hide", message = FALSE----
fits_biphasic_saem_tc_dfop_sfo <- mclapply(1:n_datasets_biphasic,
  function(i) {
    message("saem DFOP-SFO fit tc ", i)
    try(
      saem(fits_biphasic_mkin_tc[[i]][2, ], transformations = "saemix",
        fail_with_errors = TRUE, test_log_parms = TRUE, quiet = TRUE),
      silent = TRUE)
  }, mc.cores = 1)


## ----mixed_biphasic_mkin, dependson = c("fit_biphasic_mkin_const", "fit_biphasic_mkin_tc")----
fits_biphasic_mixed_const <- lapply(fits_biphasic_mkin_const, function(f) mixed(f["DFOP-SFO", ]))
fits_biphasic_mixed_tc <- lapply(fits_biphasic_mkin_tc, function(f) mixed(f["DFOP-SFO", ]))


## ----ep_biphasic, dependson = c("fit_biphasic_saem_const_sfo_sfo", "fit_biphasic_saem_const_dfop_sfo", "fit_biphasic_saem_tc_sfo_sfo", "fit_biphasic_saem_tc_dfop_sfo")----
ep_biphasic_mkin_const_sfo_sfo <- t(sapply(fits_biphasic_mkin_const,
  function(f) backtransform_odeparms(mean_degparms(f[1, ]), SFO_SFO)))
ep_biphasic_mkin_const_dfop_sfo <- t(sapply(fits_biphasic_mkin_const,
  function(f) backtransform_odeparms(mean_degparms(f[2, ]), DFOP_SFO)))
ep_biphasic_mkin_tc_sfo_sfo <- t(sapply(fits_biphasic_mkin_tc,
  function(f) backtransform_odeparms(mean_degparms(f[1, ]), SFO_SFO)))
ep_biphasic_mkin_tc_dfop_sfo <- t(sapply(fits_biphasic_mkin_tc,
  function(f) backtransform_odeparms(mean_degparms(f[2, ]), DFOP_SFO)))
get_saem_parms <- function(f, model, n_parms) {
    if (inherits(f, "try-error")) return(rep(NA, n_parms))
    parms <- f$so@results@fixed.effects
    names(parms) <- f$so@results@name.fixed
    if (is.null(f$transformations)) { # we did not have this yet
      result <- backtransform_odeparms(parms, model)
    } else {
      if (f$transformations == "mkin") {
        result <- backtransform_odeparms(parms, model)
      } else {
        result <- parms
      }
    }

    if (model$spec$parent$type == "DFOP") {
      # Make sure k1 > k2
      if (result["k1"] < result["k2"]) {
        k1_old <- result["k1"]
        k2_old <- result["k2"]
        g_old <- result["g"]
        result["k1"] <- k2_old
        result["k2"] <- k1_old
        result["g"] <- 1 - g_old
      }
    }
    return(result)
}
ep_biphasic_saem_const_sfo_sfo <- t(sapply(fits_biphasic_saem_const_sfo_sfo,
    get_saem_parms, SFO_SFO, 4))
ep_biphasic_saem_const_dfop_sfo <- t(sapply(fits_biphasic_saem_const_dfop_sfo,
    get_saem_parms, DFOP_SFO, 6))
ep_biphasic_saem_tc_sfo_sfo <- t(sapply(fits_biphasic_saem_tc_sfo_sfo,
    get_saem_parms, SFO_SFO, 4))
ep_biphasic_saem_tc_dfop_sfo <- t(sapply(fits_biphasic_saem_tc_dfop_sfo,
    get_saem_parms, DFOP_SFO, 6))
ep_biphasic_all_sfo_sfo <- list(
  mean_parms_biphasic_in,
  ep_biphasic_mkin_const_sfo_sfo, ep_biphasic_mkin_tc_sfo_sfo,
  ep_biphasic_saem_const_sfo_sfo, ep_biphasic_saem_tc_sfo_sfo)
names(ep_biphasic_all_sfo_sfo) <- c("input",
  "sep cst", "sep tc",
  "saem cst", "saem tc")
ep_biphasic_all_dfop_sfo <- list(
  mean_parms_biphasic_in,
  ep_biphasic_mkin_const_dfop_sfo, ep_biphasic_mkin_tc_dfop_sfo,
  ep_biphasic_saem_const_dfop_sfo, ep_biphasic_saem_tc_dfop_sfo)
names(ep_biphasic_all_dfop_sfo) <- c("input",
  "sep cst", "sep tc",
  "saem cst", "saem tc")


## ----show_ep_biphasic, results = "asis", size = "small", dependson = "ep_biphasic"----
show_ep_biphasic_sfo <- function(i) {
  parm_names <- c("parent_0", "k_parent", "f_parent_to_m1", "k_m1")
  ep_show <- t(sapply(ep_biphasic_all_sfo_sfo, function(p) {
      if (ncol(p) == 4) p[i, parm_names] else c(p[i, "parent_0"], NA, p[i, c("f_parent_to_m1", "k_m1")])
    }))
  aic_show <- rep("", 5)
  aic_const <- try(AIC(fits_biphasic_saem_const_sfo_sfo[[i]]$so), silent = TRUE)
  aic_show[4] <- ifelse (inherits(aic_const, "try-error"), "n.a.", round(aic_const, 1))
  aic_tc <- try(AIC(fits_biphasic_saem_tc_sfo_sfo[[i]]$so), silent = TRUE)
  aic_show[5] <- ifelse (inherits(aic_tc, "try-error"), "n.a.", round(aic_tc, 1))
  jrwb_table(signif(as.data.frame(ep_show), 3),
    caption = paste0("SFO-SFO population parameters for DFOP-SFO dataset ", i),
    label = paste0("tab:ep_biphasic_sfo_", i),
    midrules = c(1, 3),
    factors = list("Evaluation" = c("Mean input", "Separate", "Separate", "saemix", "saemix"),
      "Error model" = c("", "const", "tc", "const", "tc"),
      AIC = aic_show),
    merge_factors = "Evaluation",
    size = "small",
    NA_string = "-",
    header = gsub("_", "\\\\_", parm_names),
    footers = ifelse(any(is.na(aic_show)),
      "n.a.: not available, overparameterisation indicated by a singular Fisher Information Matrix", "")
  )
}

show_ep_biphasic_dfop <- function(i) {
  parm_names <- c("parent_0", "k1", "k2", "g", "f_parent_to_m1", "k_m1")
  ep_show <- t(sapply(ep_biphasic_all_dfop_sfo, function(p) p[i, parm_names]))
  aic_show <- rep("", 5)
  aic_const <- try(AIC(fits_biphasic_saem_const_dfop_sfo[[i]]$so), silent = TRUE)
  aic_show[4] <- ifelse (inherits(aic_const, "try-error"), "n.a.", round(aic_const, 1))
  aic_tc <- try(AIC(fits_biphasic_saem_tc_dfop_sfo[[i]]$so), silent = TRUE)
  aic_show[5] <- ifelse (inherits(aic_tc, "try-error"), "n.a.", round(aic_tc, 1))
  jrwb_table(signif(as.data.frame(ep_show), 3),
    caption = paste0("DFOP-SFO population parameters for DFOP-SFO dataset ", i),
    label = paste0("tab:ep_biphasic_dfop_", i),
    midrules = c(1, 3),
    factors = list("Evaluation" = c("Mean input", "Separate", "Separate", "saemix", "saemix"),
      "Error model" = c("", "const", "tc", "const", "tc"),
      AIC = aic_show),
    merge_factors = "Evaluation",
    size = "small",
    NA_string = "-",
    header = gsub("_", "\\\\_", parm_names),
    footers = ifelse(any(is.na(aic_show)),
      "n.a.: not available, overparameterisation indicated by a singular Fisher Information Matrix", "")
  )
}


## ----mixed_biphasic_saem_tc_sfo_1, fig.cap = "SFO-SFO saemix fit with two-component variance for DFOP-SFO dataset 1", fig.height = 6.5, dependson = "fit_biphasic_saem_tc_dfop_sfo"----
plot(fits_biphasic_saem_tc_sfo_sfo[[1]], xlab = "Time [days]")



## ----ep_biphasic_1_sfo, results = "asis", size = "small", dependson = "show_ep_biphasic"----
show_ep_biphasic_sfo(1)


## ----mixed_biphasic_saem_tc_dfop_1, fig.cap = "DFOP-SFO saemix fit with two-component variance for DFOP-SFO dataset 1", fig.height = 6.5, dependson = "fit_biphasic_saem_tc_dfop_sfo"----
plot(fits_biphasic_saem_tc_dfop_sfo[[1]], xlab = "Time [days]")



## ----ep_biphasic_1_dfop, results = "asis", size = "small", dependson = "show_ep_biphasic"----
show_ep_biphasic_dfop(1)


## ----mixed_biphasic_saem_tc_sfo_2, fig.cap = "SFO-SFO saemix fit with two-component variance for DFOP-SFO dataset 2", fig.height = 6.5, dependson = "fit_biphasic_saem_tc_dfop_sfo"----
plot(fits_biphasic_saem_tc_sfo_sfo[[2]], xlab = "Time [days]")



## ----ep_biphasic_2_sfo, results = "asis", size = "small", dependson = "show_ep_biphasic"----
show_ep_biphasic_sfo(2)


## ----mixed_biphasic_saem_tc_dfop_2, fig.cap = "DFOP-SFO saemix fit with two-component variance for DFOP-SFO dataset 2", fig.height = 6.5, dependson = "fit_biphasic_saem_tc_dfop_sfo"----
plot(fits_biphasic_saem_tc_dfop_sfo[[2]], xlab = "Time [days]")



## ----ep_biphasic_2_dfop, results = "asis", size = "small", dependson = "show_ep_biphasic"----
show_ep_biphasic_dfop(2)


## ----mixed_biphasic_saem_tc_sfo_3, fig.cap = "SFO-SFO saemix fit with two-component variance for DFOP-SFO dataset 3", fig.height = 6.5, dependson = "fit_biphasic_saem_tc_dfop_sfo"----
plot(fits_biphasic_saem_tc_sfo_sfo[[3]], xlab = "Time [days]")



## ----ep_biphasic_3_sfo, results = "asis", size = "small", dependson = "show_ep_biphasic"----
show_ep_biphasic_sfo(3)


## ----mixed_biphasic_saem_const_dfop_3, fig.cap = "DFOP-SFO saemix fit with constant variance for DFOP-SFO dataset 3", fig.height = 6.5, dependson = "fit_biphasic_saem_tc_dfop_sfo"----
plot(fits_biphasic_saem_const_dfop_sfo[[3]], xlab = "Time [days]")



## ----ep_biphasic_3_dfop, results = "asis", size = "small", dependson = "show_ep_biphasic"----
show_ep_biphasic_dfop(3)


## ----aic_stats_biphasic, dependson = "ep_biphasic", results = "hide"----------
aic_lowest_biphasic_mkin <- matrix("", nrow = n_datasets_biphasic, ncol =  n_groups_biphasic)
aic_biphasic_saem <- matrix(numeric(), nrow = n_datasets_biphasic, ncol = 4)
aic_lowest_biphasic_saem <- character(n_datasets_biphasic)
variants_biphasic_saem <- c("SFO-SFO const", "DFOP-SFO const",
  "SFO-SFO tc", "DFOP-SFO tc")
colnames(aic_biphasic_saem) <- variants_biphasic_saem
for (i in 1:n_datasets_biphasic) {
  for (j in 1:n_groups_biphasic) {
    AIC_mkin <- c(
      "SFO-SFO const" = AIC(fits_biphasic_mkin_const[[i]][["SFO-SFO", j]]),
      "DFOP-SFO const" = AIC(fits_biphasic_mkin_const[[i]][["DFOP-SFO", j]]),
      "SFO-SFO tc" = AIC(fits_biphasic_mkin_tc[[i]][["SFO-SFO", j]]),
      "DFOP-SFO tc" = AIC(fits_biphasic_mkin_tc[[i]][["DFOP-SFO", j]])
    )
    aic_lowest_biphasic_mkin[i, j] <- names(which.min(AIC_mkin))
  }
  AIC_biphasic_saem <- c(
    "SFO-SFO const" = AIC(fits_biphasic_saem_const_sfo_sfo[[i]]$so),
    "DFOP-SFO const" = tryCatch(AIC(fits_biphasic_saem_const_dfop_sfo[[i]]$so), error = function(x) NA),
    "SFO-SFO tc" = tryCatch(AIC(fits_biphasic_saem_tc_sfo_sfo[[i]]$so), error = function(x) NA),
    "DFOP-SFO tc" = tryCatch(AIC(fits_biphasic_saem_tc_dfop_sfo[[i]]$so), error = function(x) NA)
  )
  aic_biphasic_saem[i, ] <- AIC_biphasic_saem
  aic_lowest_biphasic_saem[i] <- names(which.min(AIC_biphasic_saem))
}
#table(aic_stats_biphasic_mkin)
#saem_aic_table <- table(aic_lowest_biphasic_saem) # 100


## ----box_biphasic_dfop_pub, fig.cap = "Selected population parameters obtained with DFOP-SFO for the biphasic datasets", fig.height = 5, dependson = "ep_biphasic"----
layout(1:2, heights = c(1.5, 1.2))

par(mar = c(2.1, 4.1, 2.1, 2.1))
boxplot(sapply(ep_biphasic_all_dfop_sfo, function(x) x[, "k2"]),
  ylab = "k2", ylim = c(0.0002, 0.06),
  col = c("grey", rep(c("white", "darkgrey"), 2)),
  at = c(1, 3, 4, 6, 7), xaxt = "n")
abline(h = 0.01, lty = 2, lwd = 1, col = "black")
legend("topleft", bty = "n", lty = 2:3, lwd = 1,
  title = "Distribution of input sample means",
  legend = c("Mode", "Quartiles"))
legend("topright", bty = "n", fill = c("white", "darkgrey"),
  title = "Error model",
  legend = c("Constant variance", "Two-component error"))
quartiles_tmp <- qlnorm(c(0.25, 0.75), log(0.01), trans_sd/sqrt(n_groups_biphasic))
abline(h = quartiles_tmp, lty = 3, lwd = 1, col = "black")

par(mar = c(4.1, 4.1, 2.1, 2.1))
boxplot(sapply(ep_biphasic_all_dfop_sfo, function(x) x[, "k_m1"]),
  ylab = "k\\_m1",
  col = c("grey", rep(c("white", "darkgrey"), 2)),
  at = c(1, 3, 4, 6, 7), xaxt = "n")
abline(h = 0.002, lty = 2, lwd = 1, col = "black")
quartiles_tmp <- qlnorm(c(0.25, 0.75), log(0.002), trans_sd/sqrt(n_groups_biphasic))
abline(h = quartiles_tmp, lty = 3, lwd = 1, col = "black")
axis(1, at = c(1, 3.5, 6.5), tick = FALSE,
  labels = c("Input sample means", "Separate fits", "saemix"))


## ----box_biphasic_dfop, fig.cap = "Population parameters obtained with DFOP-SFO for the biphasic datasets", fig.height = 9, dependson = "ep_biphasic"----
layout(1:6, heights = c(1.5, rep(1, 4), 1.2))

par(mar = c(2.1, 4.1, 2.1, 2.1))
boxplot(sapply(ep_biphasic_all_dfop_sfo, function(x) x[, "parent_0"]),
  ylab = "parent\\_0", ylim = c(97, 107),
  col = c("grey", rep(c("white", "darkgrey"), 2)),
  at = c(1, 3, 4, 6, 7), xaxt = "n")
legend("topleft", bty = "n", lty = 2:3, lwd = 1,
  title = "Distribution of input sample means",
  legend = c("Mode", "Quartiles"))
legend("topright", bty = "n", fill = c("white", "darkgrey"),
  title = "Error model",
  legend = c("Constant variance", "Two-component error"))
abline(h = 100, lty = 2, lwd = 1, col = "black")
quartiles_tmp <- qnorm(c(0.25, 0.75), 100, 2/sqrt(n_groups_biphasic))
abline(h = quartiles_tmp, lty = 3, lwd = 1, col = "black")

par(mar = c(2.1, 4.1, 2.1, 2.1))
boxplot(sapply(ep_biphasic_all_dfop_sfo, function(x) x[, "k1"]),
  ylab = "k1", ylim = c(0.0005, 0.15),
  col = c("grey", rep(c("white", "darkgrey"), 2)),
  at = c(1, 3, 4, 6, 7), xaxt = "n")
abline(h = 0.05, lty = 2, lwd = 1, col = "black")
quartiles_tmp <- qlnorm(c(0.25, 0.75), log(0.05), trans_sd/sqrt(n_groups_biphasic))
abline(h = quartiles_tmp, lty = 3, lwd = 1, col = "black")

boxplot(sapply(ep_biphasic_all_dfop_sfo, function(x) x[, "k2"]),
  ylab = "k2", ylim = c(0.0002, 0.03),
  col = c("grey", rep(c("white", "darkgrey"), 2)),
  at = c(1, 3, 4, 6, 7), xaxt = "n")
abline(h = 0.01, lty = 2, lwd = 1, col = "black")
quartiles_tmp <- qlnorm(c(0.25, 0.75), log(0.01), trans_sd/sqrt(n_groups_biphasic))
abline(h = quartiles_tmp, lty = 3, lwd = 1, col = "black")
boxplot(sapply(ep_biphasic_all_dfop_sfo, function(x) x[, "g"]),
  ylab = "g",
  at = c(1, 3, 4, 6, 7), xaxt = "n")
abline(h = 0.5, lty = 2, lwd = 1, col = "black")
quartiles_tmp <- plogis(qnorm(c(0.25, 0.75), qlogis(0.5), trans_sd/(sqrt(n_groups_biphasic))))
abline(h = quartiles_tmp, lty = 3, lwd = 1, col = "black")
boxplot(sapply(ep_biphasic_all_dfop_sfo, function(x) x[, "f_parent_to_m1"]),
  ylab = "f\\_parent\\_to\\_m1",
  col = c("grey", rep(c("white", "darkgrey"), 2)),
  at = c(1, 3, 4, 6, 7), xaxt = "n")
abline(h = 0.5, lty = 2, lwd = 1, col = "black")
abline(h = quartiles_tmp, lty = 3, lwd = 1, col = "black")

par(mar = c(4.1, 4.1, 2.1, 2.1))
boxplot(sapply(ep_biphasic_all_dfop_sfo, function(x) x[, "k_m1"]),
  ylab = "k\\_m1",
  col = c("grey", rep(c("white", "darkgrey"), 2)),
  at = c(1, 3, 4, 6, 7), xaxt = "n")
abline(h = 0.002, lty = 2, lwd = 1, col = "black")
quartiles_tmp <- qlnorm(c(0.25, 0.75), log(0.002), trans_sd/sqrt(n_groups_biphasic))
abline(h = quartiles_tmp, lty = 3, lwd = 1, col = "black")
axis(1, at = c(1, 3.5, 6.5), tick = FALSE,
  labels = c("Input sample means", "Separate fits", "saemix"))



## ----d24_ds, echo = TRUE------------------------------------------------------
d24_ds <- lapply(1:5, function(i) {
  subset(D24_2014$ds[[i]]$data, time <= 120, c("name", "time", "value"))
})
names(d24_ds) <- sapply(D24_2014$ds, function(ds) ds$title)
d24_ds_norm <- lapply(1:5, function(i) {
  ds_i <- subset(D24_2014$ds[[i]]$data, time <= 120, c("name", "time", "value"))
  ds_i$time <- ds_i$time * D24_2014$f_time_norm[i]
  ds_i
})
names(d24_ds_norm) <- sapply(D24_2014$ds, function(ds) ds$title)


## ----d24_separate, dependson = "d24_ds"---------------------------------------
f_d24_parent <- mmkin(c("SFO", "FOMC", "DFOP"), d24_ds, quiet = TRUE)


## ----d24_parent_sfo, fig.cap = "SFO parent kinetics in the datasets for 2,4-D", dependson = "d24_separate", fig.height = 9----
plot(f_d24_parent[1, ], main = "", legend = FALSE, xlab = "Time [days]")


## ----d24_parent_pars, results = "asis", dependson = "d24_separate"------------
jrwb_mmkin_parameters(f_d24_parent[1, ], label = "tab:d24_parent_pars", docx_in = NULL)


## ----d24_mixed, dependson = "d24_ds"------------------------------------------
f_d24_parent_norm <- mmkin(c("SFO", "FOMC"), d24_ds_norm, quiet = TRUE)
f_d24_parent_norm_tc <- update(f_d24_parent_norm, error_model = "tc")

f_d24_sfo_nlme <- nlme(f_d24_parent_norm["SFO", ])
#f_d24_fomc_nlme <- nlme(f_d24_parent_norm["FOMC", ]) # no convergence
f_d24_sfo_nlme_tc <- nlme(f_d24_parent_norm_tc["SFO", ])

f_d24_sfo_saem <- saem(f_d24_parent_norm[1, ], quiet = TRUE)
f_d24_sfo_saem_tc <- saem(f_d24_parent_norm_tc[1, ], quiet = TRUE)
f_d24_fomc_saem <- saem(f_d24_parent_norm[2, ], quiet = TRUE)


## ----d24_mixed_anova----------------------------------------------------------
anova(f_d24_sfo_nlme, f_d24_sfo_nlme_tc)


## ----d24_mixed_compare--------------------------------------------------------
compare.saemix(f_d24_sfo_saem$so, f_d24_sfo_saem_tc$so, f_d24_fomc_saem$so)


## ----d24_sfo_nlme, fig.cap = "Mixed model SFO kinetics in the datasets for 2,4-D fitted by nlme", dependson = "d24_mixed", fig.height = 4----
plot(f_d24_sfo_nlme, xlab = "Time [days]")


## ----d24_sfo_saem, fig.cap = "Mixed model SFO kinetics in the datasets for 2,4-D fitted by saemix", dependson = "d24_mixed", fig.height = 4----
plot(f_d24_sfo_saem, xlab = "Time [days]")


## ----d24_obs------------------------------------------------------------------
m_d24 = mkinmod(D24 = mkinsub("SFO", to = "DCP"),
   DCP = mkinsub("SFO", to = "DCA"),
   DCA = mkinsub("SFO"), quiet = TRUE,
   name = "m_d24", dll_dir = "mkin_dlls",
   unload = TRUE, overwrite = TRUE
)
m_d24_2 = mkinmod(D24 = mkinsub("SFO", to = "DCP"),
   DCP = mkinsub("SFO", to = "DCA", sink = FALSE),
   DCA = mkinsub("SFO"), quiet = TRUE,
   name = "m_d24_2", dll_dir = "mkin_dlls",
   unload = TRUE, overwrite = TRUE
)

f_d24_obs <- mmkin(list(
    "SFO-SFO-SFO" = m_d24,
    "SFO-SFO(ns)-SFO" = m_d24_2), d24_ds,
  error_model = "obs",
  quiet = TRUE)
f_d24_tc <- update(f_d24_obs, error_model = "tc")


## ----d24_2, fig.cap = "SFO-SFO-SFO fit for 2,4-D, Fayette", dependson = "d24_obs", fig.height = 7----
plot_sep(f_d24_obs[[1, 2]], main = "", lpos = c("topright", "bottomright", "bottomright"), xlab = "Time [days]")


## ----d24_2_2, fig.cap = "SFO-SFO(ns)-SFO fit for 2,4-D, Fayette", dependson = "d24_obs", fig.height = 7----
plot_sep(f_d24_obs[[2, 2]], main = "", lpos = c("topright", "bottomright", "bottomright"), xlab = "Time [days]")


## ----d24_2_pars, results = "asis", dependson = "d24_obs"----------------------
jrwb_mmkin_parameters(f_d24_obs[, 2],
  label = "tab:d24_2_pars", docx_in = NULL)


## ----d24_2_AIC, dependson = "d24_obs"-----------------------------------------
AIC(
  f_d24_obs[["SFO-SFO-SFO", 2]], f_d24_obs[["SFO-SFO(ns)-SFO", 2]],
  f_d24_tc[["SFO-SFO-SFO", 2]], f_d24_tc[["SFO-SFO(ns)-SFO", 2]])


## ----d24_2_lrtest, size = "small", dependson = "d24_obs"----------------------
i = 2
lrtest(f_d24_obs[["SFO-SFO(ns)-SFO", i]], f_d24_tc[["SFO-SFO(ns)-SFO", i]])
lrtest(f_d24_obs[, i])


## ----d24_3, fig.cap = "SFO-SFO-SFO fit for 2,4-D, RefSol 03-G", dependson = "d24_obs", fig.height = 7----
plot_sep(f_d24_obs[[1, 3]], main = "", lpos = c("topright", "bottomright", "bottomright"))


## ----d24_3_pars, results = "asis", dependson = "d24_obs"----------------------
jrwb_mmkin_parameters(f_d24_obs[1, 3],
  label = "tab:d24_3_pars", docx_in = NULL)


## ----d24_3_AIC, dependson = "d24_obs"-----------------------------------------
AIC(
  f_d24_obs[["SFO-SFO-SFO", 3]], f_d24_obs[["SFO-SFO(ns)-SFO", 3]],
  f_d24_tc[["SFO-SFO-SFO", 3]], f_d24_tc[["SFO-SFO(ns)-SFO", 3]])


## ----d24_3_lrtest, size = "small", dependson = "d24_obs"----------------------
i = 3
lrtest(f_d24_obs[["SFO-SFO-SFO", i]], f_d24_tc[["SFO-SFO-SFO", i]])
lrtest(f_d24_obs[, i])


## ----d24_4, fig.cap = "SFO-SFO-SFO fit for 2,4-D, Site E1", dependson = "d24_obs", fig.height = 7----
plot_sep(f_d24_obs[[1, 4]], main = "", lpos = c("topright"), xlab = "Time [days]")


## ----d24_4_pars, results = "asis", dependson = "d24_obs"----------------------
jrwb_mmkin_parameters(f_d24_obs[1:2, 4],
  label = "tab:d24_4_pars", docx_in = NULL)


## ----d24_4_2, fig.cap = "SFO-SFO(ns)-SFO fit for 2,4-D, Site E1", dependson = "d24_obs", fig.height = 7----
plot_sep(f_d24_obs[[2, 4]], main = "", lpos = c("topright"), xlab = "Time [days]")


## ----d24_4_AIC, dependson = "d24_obs"-----------------------------------------
AIC(
  f_d24_obs[["SFO-SFO-SFO", 4]], f_d24_obs[["SFO-SFO(ns)-SFO", 4]],
  f_d24_tc[["SFO-SFO-SFO", 4]], f_d24_tc[["SFO-SFO(ns)-SFO", 4]])


## ----d24_4_lrtest, size = "small", dependson = "d24_obs"----------------------
i = 4
lrtest(f_d24_obs[["SFO-SFO-SFO", i]], f_d24_tc[["SFO-SFO-SFO", i]])
lrtest(f_d24_obs[, i])


## ----d24_5, fig.cap = "SFO-SFO-SFO fit for 2,4-D, Site I2", dependson = "d24_obs", fig.height = 7----
plot_sep(f_d24_obs[[1, 5]], main = "", lpos = c("topright"), xlab = "Time [days]")


## ----d24_5_pars, results = "asis", dependson = "d24_obs"----------------------
jrwb_mmkin_parameters(f_d24_obs[1, 5],
  label = "tab:d24_5_pars", docx_in = NULL)


## ----d24_5_AIC, dependson = "d24_obs"-----------------------------------------
AIC(
  f_d24_obs[["SFO-SFO-SFO", 5]], f_d24_obs[["SFO-SFO(ns)-SFO", 5]],
  f_d24_tc[["SFO-SFO-SFO", 5]], f_d24_tc[["SFO-SFO(ns)-SFO", 5]])


## ----d24_5_lrtest, size = "small", dependson = "d24_obs"----------------------
i = 5
lrtest(f_d24_obs[["SFO-SFO-SFO", i]], f_d24_tc[["SFO-SFO-SFO", i]])
lrtest(f_d24_obs[, i])


## ----d24_norm_mmkin, dependson = "d24_ds"-------------------------------------
f_d24_norm_const <- mmkin(list(
    "SFO-SFO-SFO" = m_d24,
    "SFO-SFO(ns)-SFO" = m_d24_2), d24_ds_norm,
  error_model = "const",
  quiet = TRUE)
f_d24_norm_obs <- update(f_d24_norm_const, error_model = "obs")
f_d24_norm_tc <- update(f_d24_norm_const, error_model = "tc")


## ----d24_norm_nlme, dependson = "d24_norm_mmkin"------------------------------
# f_d24_nlme_const <- nlme(f_d24_norm_const[1, ],
#   control = list(pnlsMaxIter = 50, msMaxIter = 200, tolerance = 2e-3),
#   verbose = TRUE) # no chance for convergence it seems
f_d24_2_nlme_const <- nlme(f_d24_norm_const[2, ])
# fast convergence with defaults with this model

#f_d24_nlme_obs <- nlme(f_d24_norm_obs[1, ], # converged before normalisation
#  control = list(pnlsMaxIter = 100, tolerance = 2e-3), verbose = TRUE)
f_d24_2_nlme_obs <- nlme(f_d24_norm_obs[2, ],
  control = list(pnlsMaxIter = 100, tolerance = 1e-4))
f_d24_nlme_tc <- nlme(f_d24_norm_tc[1, ],
  control = list(pnlsMaxIter = 100, tolerance = 2e-3))
f_d24_2_nlme_tc <- nlme(f_d24_norm_tc[2, ],
  control = list(pnlsMaxIter = 100, tolerance = 2e-3))
# f_d24_2_nlme_tc <- nlme(f_d24_norm_tc[2, ],
#   control = list(pnlsMaxIter = 20, tolerance = 2e-3),
#   verbose = TRUE)


## ----d24_nlme_anova, dependson = "d24_norm_nlme"------------------------------
anova(f_d24_2_nlme_const,
  f_d24_2_nlme_obs,
  f_d24_nlme_tc, f_d24_2_nlme_tc)


## ----d24_2_nlme_const, fig.cap = "SFO-SFO(ns)-SFO fitted using nlme assuming constant error", fig.height = 7, dependson = "d24_norm_nlme"----
plot(f_d24_2_nlme_const, xlab = "Time [days]")


## ----d24_2_nlme_obs, fig.cap = "SFO-SFO-SFO(ns) fitted using nlme assuming variance by variable", fig.height = 7, dependson = "d24_norm_nlme"----
plot(f_d24_2_nlme_obs, xlab = "Time [days]")


## ----d24_2_nlme_tc, fig.cap = "SFO-SFO(ns)-SFO fitted using nlme assuming two-component error", fig.height = 7, dependson = "d24_norm_nlme"----
plot(f_d24_2_nlme_tc, xlab = "Time [days]")


## ----d24_saem_const, dependson = "d24_norm_mmkin"-----------------------------
#f_d24_saem_const <- saem(f_d24_norm_const[1, ], test_log_parms = TRUE, quiet = TRUE) # Error in FIM inversion


## ----d24_saem_tc, dependson = "d24_norm_mmkin"--------------------------------
f_d24_saem_tc <- saem(f_d24_norm_tc[1, ], test_log_parms = TRUE, quiet = TRUE)


## ----d24_saem_tc_convergence, fig.cap = "Convergence plot for SFO-SFO-SFO fitted using saemix assuming two-component error", fig.height = 8, dependson = "d24_saem_tc", dev = "pdf", results = "hide"----
par(las = 0)
saemix::plot(f_d24_saem_tc$so, plot.type = "convergence")


## ----d24_saem_tc_plot, fig.cap = "SFO-SFO-SFO fitted using saemix assuming two-component error", fig.height = 7, dependson = "d24_saem_tc"----
plot(f_d24_saem_tc, xlab = "Time [days]")


## ----d24_saem_tc_plot_xlim, fig.cap = "SFO-SFO-SFO fitted using saemix assuming two-component error", fig.height = 7, dependson = "d24_saem_tc"----
plot(f_d24_saem_tc, xlab = "Time [days]", xlim = c(0, 30))


## ----d24_2_saem_const, dependson = "d24_norm_mmkin"---------------------------
f_d24_2_saem_const <- saem(f_d24_norm_const[2, ], test_log_parms = TRUE, quiet = TRUE)


## ----d24_2_saem_tc, dependson = "d24_norm_mmkin"------------------------------
f_d24_2_saem_tc <- saem(f_d24_norm_tc[2, ], test_log_parms = TRUE, quiet = TRUE)


## ----d24_2_saem_const_convergence, fig.cap = "Convergence plot for SFO-SFO(ns)-SFO fitted using saemix assuming constant error", fig.height = 8, dependson = "d24_2_saem_const", dev = "pdf", results = "hide"----
par(las = 0)
saemix::plot(f_d24_2_saem_const$so, plot.type = "convergence")


## ----d24_2_saem_const_plot, fig.cap = "SFO-SFO(ns)-SFO fitted using saemix assuming constant error", fig.height = 7, dependson = "d24_2_saem_const"----
plot(f_d24_2_saem_const, xlab = "Time [days]", xlim = c(0, 30))


## ----d24_2_saem_tc_convergence, fig.cap = "Convergence plot for SFO-SFO(ns)-SFO fitted using saemix assuming two-component error", fig.height = 8, dependson = "d24_2_saem_tc", dev = "pdf", results = "hide"----
par(las = 0)
saemix::plot(f_d24_2_saem_tc$so, plot.type = "convergence")


## ----d24_2_saem_tc_plot, fig.cap = "SFO-SFO(ns)-SFO fitted using saemix assuming two-component error", fig.height = 7, dependson = "d24_2_saem_tc"----
plot(f_d24_2_saem_tc, xlab = "Time [days]", xlim = c(0, 30))


## ----d24_saem_comparison, dependson = c("d24_saem_2_const", "d24_saem_2_tc")----
compare.saemix(f_d24_saem_tc$so, f_d24_2_saem_const$so, f_d24_2_saem_tc$so)


## ----biphasic_transformations-------------------------------------------------
f_tmp <- fits_biphasic_saem_tc_dfop_sfo[[1]]
map_back <- f_tmp$so@results@map.psi
map_trans_saemix <- f_tmp$so@results@map.phi
names(map_back) <- f_tmp$so@results@name.fixed
map_trans <- as.data.frame(t(sapply(1:8, function(i) {
  res <- transform_odeparms(as.matrix(map_back)[i, ],
    f_tmp$mkinmod,
    transform_rates = TRUE, transform_fractions = TRUE)
  return(res)
})))

pdf("Reviewer_2_reply_Figure_1.pdf", height = 9)
par(mfrow = c(5, 2))
par(mar = c(5.1, 4.1, 2.1, 2.1))
for (i in 2:6) {
  qqnorm(map_back[, i], main = "")
  qqline(map_back[, i])
  legend("topleft", inset = c(0.05, 0.05), bty = "n", legend = c(names(map_back)[i],
      paste0("p = ", signif(shapiro.test(map_back[, i])$p.value, 2))))
  qqnorm(map_trans[, i], main = "")
  qqline(map_trans[, i])
  legend("topleft", inset = c(0.05, 0.05), bty = "n", legend = c(names(map_trans)[i],
      paste0("p = ", signif(shapiro.test(map_trans[, i])$p.value, 2))))
}
dev.off()


## ----d24_transformations------------------------------------------------------
map_trans <- f_d24_saem_tc$so@results@map.phi
names(map_trans) <- f_d24_saem_tc$so@results@name.fixed
map_back <- as.data.frame(t(sapply(1:5, function(i) {
  res <- backtransform_odeparms(as.matrix(map_trans)[i, ],
    f_d24_saem_tc$mkinmod,
    transform_rates = TRUE, transform_fractions = TRUE)
  return(res)
})))

pdf("Reviewer_2_reply_Figure_2.pdf", height = 9)
par(mfrow = c(5, 2))
par(mar = c(5.1, 4.1, 2.1, 2.1))
for (i in 2:6) {
  qqnorm(map_back[, i], main = "")
  qqline(map_back[, i])
  legend("topleft", inset = c(0.05, 0.05), bty = "n", legend = c(names(map_back)[i],
      paste0("p = ", signif(shapiro.test(map_back[, i])$p.value, 2))))
  qqnorm(map_trans[, i], main = "")
  qqline(map_trans[, i])
  legend("topleft", inset = c(0.05, 0.05), bty = "n", legend = c(names(map_trans)[i],
      paste0("p = ", signif(shapiro.test(map_trans[, i])$p.value, 2))))
}
dev.off()


## ----d24_ep_sep, results = "asis", cache = FALSE------------------------------
d24_ep <- data.frame(
  ep_type = c(
    "\\DTfifty", "ff",
    "\\DTfifty", "ff",
    "\\DTfifty"),
  compound = c("2,4-D", "2,4-D", "DCP", "DCP", "DCA"),
  target = c(NA, "DCP", NA, "DCA", NA))
d24_ep$eu = c(4.4, 1, 7.0, NA, 10.4)

f_sep <- list(
  f_d24_parent[[1, 1]],
  f_d24_obs[[2, 2]],
  f_d24_obs[[1, 3]],
  f_d24_obs[[2, 4]],
  f_d24_obs[[1, 5]])

ep_sep <- matrix(NA, 5, 5)
ep_sep[, 1] <- sapply(f_sep, function(f) endpoints(f)$distimes[["D24", "DT50"]])
ep_sep[2:5, 2] <- sapply(f_sep[2:5], function(f) endpoints(f)$ff[["D24_DCP"]])
ep_sep[2:5, 3] <- sapply(f_sep[2:5], function(f) endpoints(f)$distimes[["DCP", "DT50"]])
ep_sep[c(2, 4), 4] <- sapply(f_sep[c(2, 4)], function(f) endpoints(f)$ff[["D24_DCP"]])
ep_sep[c(3, 5), 4] <- 1
ep_sep[2:5, 5] <- sapply(f_sep[2:5], function(f) endpoints(f)$distimes[["DCA", "DT50"]])

ep_sep_ff_mean <- apply(ep_sep[, c(2, 4)], 2, mean, na.rm = TRUE)

remove_NA <- function(df) {
  ret <- list()
  cns <- colnames(df)
  for (cn in cns) {
    ret[[cn]] <- gsub("NA", "  ", df[[cn]])
  }
  data.frame(ret)
}
ep_sep_show <- remove_NA(format(as.data.frame(ep_sep), digits = 4))

jrwb_table(ep_sep_show,
  caption = "Non-normalised endpoints for modelling as obtained in the separate fits to the data",
  label = "tab:d24_ep_sep",
  factors = list(Soil = names(d24_ds)),
  units = c("d", "-", "d", "-", "d"),
  group_names = c("2,4-D", "DCP", "DCA"),
  group_sizes = c(2, 2, 1),
  header = c("\\DTfifty", "ff to DCP", "\\DTfifty", "ff to DCA", "\\DTfifty"),
  annot = list(
    a = list(V5 = 2),
    b = list(V4 = c(3, 5))),
  summary_lines = list("Arithmetic mean" =
    list(vars = c("V2", "V4"), fun = function(x) round(mean(as.numeric(x), na.rm = TRUE), 3))),
  footer = c(
    "ff: Formation fraction",
    "$^a$ Based on a non-significant rate constant",
    "$^b$ Pathway from DCP to sink was negligible")
)


## ----d24_ep_sep_norm, results = "asis", cache = FALSE-------------------------
ep_sep_norm <- apply(ep_sep[, c(1, 3, 5)], 2, function(x) x * D24_2014$f_time_norm)
ep_sep_norm_geo <- apply(ep_sep_norm, 2, geomean, na.rm = TRUE)
ep_sep_norm_show <- remove_NA(format(as.data.frame(ep_sep_norm), digits = 4))

jrwb_table(ep_sep_norm_show,
  caption = "Normalised modelling \\DTfifty values as obtained in the separate fits to the data",
  label = "tab:d24_ep_sep_norm",
  factors = list(Soil = names(d24_ds)),
  units = c("d", "d", "d"),
  header = c("2,4-D", "DCP", "DCA"),
  annot = list(a = list(V3 = 2)),
  summary_lines = list("Geometric mean" = list(vars = c("V1", "V2", "V3"),
      fun = function(x) signif(geomean(as.numeric(x), na.rm = TRUE), 3))),
  footer = c("$^a$ Based on a non-significant rate constant")
)


## ----ep_mixed, results = "asis", dependson = c("d24_norm_nlme", "d24_saem_tc")----
ep_nlme <- endpoints(f_d24_2_nlme_obs)
ep_saem <- endpoints(f_d24_2_saem_tc)

ep_mix <- matrix(NA, 5, 2)
ep_mix[1:3, 1] <- ep_nlme$distimes[, 1]
ep_mix[1:3, 2] <- ep_saem$distimes[, 1]
ep_mix[4, 1] <- ep_nlme$ff[1]
ep_mix[4, 2] <- ep_saem$ff[1]
ep_mix[5, ] <- c(1, 1)
ep_mix_show <- format(as.data.frame(ep_mix), digits = 3)

jrwb_table(t(ep_mix_show)[, c(1, 4, 2, 5, 3)],
  caption = "Modelling endpoints derived from mixed effects models fitted to the pathway",
  label = "tab:d24_ep_mixed",
  header = c("\\DTfifty", "ff to DCP", "\\DTfifty", "ff to DCA", "\\DTfifty"),
  units = c("d", "-", "d", "-", "d"),
  group_names = c("2,4-D", "DCP", "DCA"),
  group_sizes = c(2, 2, 1),
  factors = list(Algorithm = c("nlme", "saemix")),
  footer = "ff: Formation fraction")


## ----d24_saem_overlay, fig.cap = "Overlay of degradation curves derived from separate fits and the best saemix fit, first 30 days", fig.height = 7.5, dependson = "f_d24_saem_tc"----
d24_sep_pred <- mkinpredict(m_d24,
  c(k_D24 = log(2)/4.42,
    f_D24_to_DCP = 0.355,
    k_DCP = log(2)/0.829,
    f_DCP_to_DCA = 0.619,
    k_DCA = log(2)/8.52),
  c(D24 =
    mean_degparms(f_d24_norm_tc[1, ])[["D24_0"]],
  DCP = 0, DCA = 0))

plot(f_d24_saem_tc, pred_over = list("Separate" = d24_sep_pred), ncol.legend = 4, xlab = "Time [days]", xlim = c(0, 30))


## ----d24_nlme_overlay, fig.cap = "Overlay of degradation curves derived from separate fits and the best nlme fit, first 30 days", fig.height = 7.5, dependson = "d24_norm_nlme"----
plot(f_d24_2_nlme_obs, pred_over = list("Separate" = d24_sep_pred), ncol.legend = 4, xlab = "Time [days]", xlim = c(0, 30))


## ----dmta_data, echo = TRUE---------------------------------------------------
dmta_ds <- lapply(1:8, function(i) {
  ds_i <- dimethenamid_2018$ds[[i]]$data
  ds_i[ds_i$name == "DMTAP", "name"] <-  "DMTA"
  ds_i$time <- ds_i$time * dimethenamid_2018$f_time_norm[i]
  ds_i
})
names(dmta_ds) <- sapply(dimethenamid_2018$ds, function(ds) ds$title)
dmta_ds[["Borstel"]] <- rbind(dmta_ds[["Borstel 1"]], dmta_ds[["Borstel 2"]])
dmta_ds[["Borstel 1"]] <- NULL
dmta_ds[["Borstel 2"]] <- NULL
dmta_ds[["Elliot"]] <- rbind(dmta_ds[["Elliot 1"]], dmta_ds[["Elliot 2"]])
dmta_ds[["Elliot 1"]] <- NULL
dmta_ds[["Elliot 2"]] <- NULL


## ----dmta_mkin, dependson = "dmta_data"---------------------------------------
m_dmta_1 <- mkinmod(
  DMTA = mkinsub("SFO", c("M23", "M27", "M31")),
  M23 = mkinsub("SFO"),
  M27 = mkinsub("SFO"),
  M31 = mkinsub("SFO"),
  name = "m_dmta_1", dll_dir = "mkin_dlls",
  unload = TRUE, overwrite = TRUE,
  quiet = TRUE
)
m_dmta_2 <- mkinmod(
  DMTA = mkinsub("SFO", c("M23", "M27", "M31")),
  M23 = mkinsub("SFO"),
  M27 = mkinsub("SFO"),
  M31 = mkinsub("SFO", "M27", sink = FALSE),
  name = "m_dmta_2", dll_dir = "mkin_dlls",
  unload = TRUE, overwrite = TRUE,
  quiet = TRUE
)
m_dmta_3 <- mkinmod(
  DMTA = mkinsub("DFOP", c("M23", "M27", "M31")),
  M23 = mkinsub("SFO"),
  M27 = mkinsub("SFO"),
  M31 = mkinsub("SFO"),
  name = "m_dmta_3", dll_dir = "mkin_dlls",
  unload = TRUE, overwrite = TRUE,
  quiet = TRUE
)
m_dmta_4 <- mkinmod(
  DMTA = mkinsub("DFOP", c("M23", "M27", "M31")),
  M23 = mkinsub("SFO"),
  M27 = mkinsub("SFO"),
  M31 = mkinsub("SFO", "M27", sink = FALSE),
  name = "m_dmta_4", dll_dir = "mkin_dlls",
  unload = TRUE, overwrite = TRUE,
  quiet = TRUE
)
f_dmta_mkin_const <- mmkin(
  list("SFO-SFO3" = m_dmta_1, "SFO-SFO3b" = m_dmta_2, "DFOP-SFO3" = m_dmta_3, "DFOP-SFO3+" = m_dmta_4),
  dmta_ds, quiet = TRUE)
f_dmta_mkin_obs <- mmkin(
  list("SFO-SFO3" = m_dmta_1, "SFO-SFO3b" = m_dmta_2, "DFOP-SFO3" = m_dmta_3, "DFOP-SFO3+" = m_dmta_4),
  dmta_ds, quiet = TRUE, error_model = "obs")
f_dmta_mkin_tc <- mmkin(
  list("SFO-SFO3" = m_dmta_1, "SFO-SFO3b" = m_dmta_2, "DFOP-SFO3" = m_dmta_3, "DFOP-SFO3+" = m_dmta_4),
  dmta_ds, quiet = TRUE, error_model = "tc")


## ----dmta_plot_mkin_const_1, fig.cap = "Separate fits of the SFO-SFO3 model to the normalised DMTA data assuming constant variance", fig.height = 8.5, dependson = "dmta_mkin"----
plot(mixed(f_dmta_mkin_const[1, ]), xlab = "Time [days]")


## ----dmta_plot_mkin_const_2, fig.cap = "Separate fits of the SFO-SFO3+ model to the normalised DMTA data assuming constant variance", fig.height = 8.5, dependson = "dmta_mkin"----
plot(mixed(f_dmta_mkin_const[2, ]), xlab = "Time [days]")


## ----dmta_mkin_aic, results = "asis", dependson = "dmta_mkin"-----------------
aic_dmta_mkin <- cbind(
  sapply(f_dmta_mkin_const[1, ], AIC),
  sapply(f_dmta_mkin_const[2, ], AIC),
  sapply(f_dmta_mkin_const[3, ], AIC),
  sapply(f_dmta_mkin_const[4, ], AIC),
  sapply(f_dmta_mkin_obs[1, ], AIC),
  sapply(f_dmta_mkin_obs[2, ], AIC),
  sapply(f_dmta_mkin_obs[3, ], AIC),
  sapply(f_dmta_mkin_obs[4, ], AIC),
  sapply(f_dmta_mkin_tc[1, ], AIC),
  sapply(f_dmta_mkin_tc[2, ], AIC),
  sapply(f_dmta_mkin_tc[3, ], AIC),
  sapply(f_dmta_mkin_tc[4, ], AIC)
)
min_indices <- apply(aic_dmta_mkin, 1, which.min)
aic_dmta_mkin_display <- round(aic_dmta_mkin, 2)
for (i in 1:6) {
  aic_dmta_mkin_display[i, min_indices[i]] <- paste0("\\textbf{",
    aic_dmta_mkin_display[i, min_indices[i]], "}")
}
jrwb_table(aic_dmta_mkin_display[, 1:4],
  caption = "AIC values for the separate fits of three degradation models to data from six soils using constant variance",
  label = "tab:aic_dmta_mkin_const",
  header = c("SFO-SFO3", "SFO-SFO3+", "DFOP-SFO3", "DFOP-SFO3+"),
  factors = list(Soil = names(dmta_ds))
)
jrwb_table(aic_dmta_mkin_display[, 5:8],
  caption = "AIC values for the separate fits of three degradation models to data from six soils using variance by variable",
  label = "tab:aic_dmta_mkin_obs",
  header = c("SFO-SFO3", "SFO-SFO3+", "DFOP-SFO3", "DFOP-SFO3+"),
  factors = list(Soil = names(dmta_ds))
)
jrwb_table(aic_dmta_mkin_display[, 9:12],
  caption = "AIC values for the separate fits of three degradation models to data from six soils using two-component error",
  label = "tab:aic_dmta_mkin_tc",
  header = c("SFO-SFO3", "SFO-SFO3+", "DFOP-SFO3", "DFOP-SFO3+"),
  factors = list(Soil = names(dmta_ds))
)


## ----dmta_plot_mkin_tc_4, fig.cap = "Separate fits of the DFOP-SFO3+ model to the normalised DMTA data assuming two-component error", fig.height = 8.5, dependson = "dmta_mkin"----
plot(mixed(f_dmta_mkin_tc[4, ]), xlab = "Time [days]")


## ----dmta_plot_mkin_tc_4_test, fig.cap = "Separate fits of the DFOP-SFO3+ model to the normalised DMTA data assuming two-component error, average curve only considering rate constants that can be distinguished from zero", fig.height = 8.5, dependson = "dmta_mkin"----
plot(mixed(f_dmta_mkin_tc[4, ]), xlab = "Time [days]", test_log_parms = TRUE)


## ----dmta_saem_1_const, dependson = "dmta_mkin"-------------------------------
f_dmta_saem_1_const <- saem(f_dmta_mkin_const[1, ], test_log_parms = TRUE, quiet = TRUE)


## ----dmta_saem_1_const_convergence, fig.cap = "Convergence plot of the saemix fit of the SFO-SFO3 model with constant variance to the normalised DMTA data", fig.height = 7.5, dev = "pdf", dependson = "dmta_saem_1_const", results = "hide"----
par(las = 0)
saemix::plot(f_dmta_saem_1_const$so, plot.type = "convergence")


## ----dmta_saem_1_const_overlay, fig.cap = "SFO-SFO3 model with constant variance fitted with saemix to the data for DMTA in six soils, with additional degradation curves derived from separate evaluations for comparison", fig.height = 8.5, dependson = c("dmta_mkin", "dmta_saem_1_const")----
dmta_parms_sep_const <- backtransform_odeparms(mean_degparms(f_dmta_mkin_const[1, ]), m_dmta_1)
dmta_pred_sep_const <- mkinpredict(m_dmta_1,
  dmta_parms_sep_const[-1],
  c(DMTA = dmta_parms_sep_const[[1]], M23 = 0, M27 = 0, M31 = 0),
  outtimes = seq(0, 250, by = 0.3))
plot(f_dmta_saem_1_const, pred_over = list("Separate fits" = dmta_pred_sep_const), xlab = "Time [days]")


## ----dmta_saem_2_const, dependson = "dmta_mkin"-------------------------------
f_dmta_saem_2_const <- saem(f_dmta_mkin_const[2, ], test_log_parms = TRUE, quiet = TRUE)


## ----dmta_saem_2_const_convergence, fig.cap = "Convergence plot of the saemix fit of the SFO-SFO3+ model with constant variance to the normalised DMTA data", fig.height = 7.5, dev = "pdf", dependson = "dmta_saem_2_const", results = "hide"----
par(las = 0)
saemix::plot(f_dmta_saem_2_const$so, plot.type = "convergence")


## ----dmta_saem_2_const_overlay, fig.cap = "SFO-SFO3+ model with constant variance fitted with saemix to the data for DMTA in six soils, with additional degradation curves derived from separate evaluations for comparison", fig.height = 8.5, dependson = c("dmta_mkin", "dmta_saem_2_const")----
dmta_parms_sep_const <- backtransform_odeparms(mean_degparms(f_dmta_mkin_const[2, ]), m_dmta_2)
dmta_pred_sep_const <- mkinpredict(m_dmta_2,
  dmta_parms_sep_const[-1],
  c(DMTA = dmta_parms_sep_const[[1]], M23 = 0, M27 = 0, M31 = 0),
  outtimes = seq(0, 250, by = 0.3))
plot(f_dmta_saem_2_const, pred_over = list("Separate fits" = dmta_pred_sep_const), xlab = "Time [days]")


## ----dmta_saem_3_const, dependson = "dmta_mkin"-------------------------------
#f_dmta_saem_3_const <- saem(f_dmta_mkin_const[3, ], quiet = TRUE, test_log_parms = TRUE)


## ----dmta_saem_3_tc, dependson = "dmta_mkin"----------------------------------
#f_dmta_saem_3_tc <- saem(f_dmta_mkin_tc[3, ], quiet = TRUE, test_log_parms = TRUE)


## ----dmta_saem_4_const, dependson = "dmta_mkin"-------------------------------
#f_dmta_saem_4_const <- saem(f_dmta_mkin_const[4, ], quiet = TRUE, test_log_parms = TRUE)


## ----dmta_saem_4_tc, dependson = "dmta_mkin"----------------------------------
f_dmta_saem_4_tc <- saem(f_dmta_mkin_tc[4, ], quiet = TRUE, test_log_parms = TRUE)


## ----dmta_saem_4_tc_convergence, fig.cap = "Convergence plot of the saemix fit of the DFOP-SFO3+ model with two-component error to the normalised DMTA data", fig.height = 7.5, dev = "pdf", dependson = "dmta_saem_4_tc", results = "hide"----
par(las = 0)
saemix::plot(f_dmta_saem_4_tc$so, plot.type = "convergence")


## ----dmta_saem_4_tc_overlay_1, fig.cap = "DFOP-SFO3+ model with two-component error fitted with saemix to the data for DMTA in six soils, with additional degradation curves derived from untested parameters from separate evaluations ", fig.height = 8.5, dependson = c("dmta_mkin", "dmta_saem_4_tc")----
dmta_parms_sep_4_tc <- backtransform_odeparms(mean_degparms(f_dmta_mkin_tc[4, ], test_log_parms = FALSE), m_dmta_4)
dmta_pred_sep_4_tc <- mkinpredict(m_dmta_4,
  dmta_parms_sep_4_tc[-1],
  c(DMTA = dmta_parms_sep_4_tc[[1]], M23 = 0, M27 = 0, M31 = 0),
  outtimes = seq(0, 250, by = 0.3))
plot(f_dmta_saem_4_tc, pred_over = list("Separate fits" = dmta_pred_sep_4_tc), xlab = "Time [days]")


## ----dmta_saem_4_tc_overlay_2, fig.cap = "DFOP-SFO3+ model with two-component error fitted with saemix to the data for DMTA in six soils, with additional degradation curves derived from tested parameters from separate evaluations ", fig.height = 8.5, dependson = c("dmta_mkin", "dmta_saem_4_tc")----
dmta_parms_sep_4_tc <- backtransform_odeparms(mean_degparms(f_dmta_mkin_tc[4, ], test_log_parms = TRUE), m_dmta_4)
dmta_pred_sep_4_tc <- mkinpredict(m_dmta_4,
  dmta_parms_sep_4_tc[-1],
  c(DMTA = dmta_parms_sep_4_tc[[1]], M23 = 0, M27 = 0, M31 = 0),
  outtimes = seq(0, 250, by = 0.3))
plot(f_dmta_saem_4_tc, pred_over = list("Separate fits" = dmta_pred_sep_4_tc), xlab = "Time [days]")


## ----dmta_nlme, dependson = "dmta_mkin", results = "hide"---------------------
f_dmta_nlme_3_obs <- nlme(f_dmta_mkin_obs[3, ],
  control = list(pnlsMaxIter = 30, tolerance = 1e-4),
  verbose = TRUE)
f_dmta_nlme_4_obs <- nlme(f_dmta_mkin_obs[4, ],
  control = list(pnlsMaxIter = 30, msMaxIter = 70, tolerance = 2e-3),
  verbose = TRUE)
f_dmta_nlme_3_tc <- nlme(f_dmta_mkin_tc[3, ],
  control = list(pnlsMaxIter = 30, tolerance = 1e-3),
  verbose = TRUE)
f_dmta_nlme_4_tc <- nlme(f_dmta_mkin_tc[4, ],
  control = list(pnlsMaxIter = 30, tolerance = 1e-3),
  verbose = TRUE)


## ----dmta_nlme_failed, dependson = "dmta_mkin", results = "hide"--------------
#f_dmta_nlme_1_const <- nlme(f_dmta_mkin_const[1, ],
#  control = list(pnlsMaxIter = 30, tolerance = 1e-3),
#  verbose = TRUE) # did not converge
#f_dmta_nlme_2_const <- nlme(f_dmta_mkin_const[2, ],
#  control = list(pnlsMaxIter = 50, tolerance = 1e-3),
#  verbose = TRUE) # did not converge


## ----dmta_saemix_comparisons, dependson = c("dmta_saem_1_const", "dmta_saem_2_const", "dmta_saem_4_const", "dmta_saem_4_tc")----
compare.saemix(f_dmta_saem_1_const$so, f_dmta_saem_2_const$so, f_dmta_saem_4_tc$so)


## ----dmta_mixed_comparisons, dependson = "dmta_nlme"--------------------------
anova(f_dmta_nlme_3_obs, f_dmta_nlme_3_tc, f_dmta_nlme_4_obs, f_dmta_nlme_4_tc)


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_parent[1:2, 1], caption_addition = "2,4-D")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_parent[1:2, 2], caption_addition = "2,4-D")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_parent[1:2, 3], caption_addition = "2,4-D")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_parent[1:2, 4], caption_addition = "2,4-D")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_parent[1:2, 5], caption_addition = "2,4-D")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_sfo_nlme, "SFO fit with nlme to 2,4-D data, constant variance", "lst:f_d24_sfo_nlme_const")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_sfo_nlme_tc, "SFO fit with nlme to 2,4-D data, two-component error", "lst:f_d24_sfo_nlme_tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_sfo_saem, "SFO fit with saemix to 2,4-D data, constant variance", "lst:f_d24_sfo_saem_const")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_sfo_saem_tc, "SFO fit with saemix to 2,4-D data, two-component error", "lst:f_d24_sfo_saem_tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_fomc_saem, "FOMC fit with saemix to 2,4-D data, constant variance", "lst:f_d24_fomc_saem_const")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_obs[1:2, 2], caption_addition = "variance by variable", label_addition = "obs")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_tc[1:2, 2], caption_addition = "two-component error", label_addition = "tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_obs[1:2, 3], caption_addition = "variance by variable", label_addition = "obs")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_tc[1:2, 3], caption_addition = "two-component error", label_addition = "tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_obs[1:2, 4], caption_addition = "variance by variable", label_addition = "obs")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_tc[1:2, 4], caption_addition = "two-component error", label_addition = "tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_obs[1:2, 5], caption_addition = "variance by variable", label_addition = "obs")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mmkin(f_d24_tc[1:2, 5], caption_addition = "two-component error", label_addition = "tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_nlme_tc, "SFO-SFO-SFO fit with nlme to 2,4-D data, two-component error", "lst:f_d24_nlme_tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_2_nlme_const, "SFO-SFO(ns)-SFO fit with nlme to 2,4-D data, constant variance", "lst:f_d24_2_nlme_const")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_2_nlme_obs, "SFO-SFO(ns)-SFO fit with nlme to 2,4-D data, variance by variable", "lst:f_d24_2_nlme_obs")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_2_nlme_tc, "SFO-SFO(ns)-SFO fit with nlme to 2,4-D data, two-component error", "lst:f_d24_2_nlme_tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_saem_tc, "SFO-SFO-SFO fit with saem to 2,4-D data, two-component error", "lst:f_d24_saem_tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_2_saem_const, "SFO-SFO(ns)-SFO fit with saem to 2,4-D data, constant variance", "lst:f_d24_2_saem_const")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_d24_2_saem_tc, "SFO-SFO(ns)-SFO fit with saem to 2,4-D data, two-component error", "lst:f_d24_2_saem_tc")


## ----results = 'asis', cache = FALSE------------------------------------------
for (i in 1:6) {
  jrwb_mmkin(f_dmta_mkin_const[1, i], caption_addition = "constant variance", label_addition = "cst")
  cat("  \\clearpage \n")
  jrwb_mmkin(f_dmta_mkin_tc[1, i], caption_addition = "two-component error", label_addition = "tc")
  cat("  \\clearpage \n")
}


## ----results = 'asis', cache = FALSE------------------------------------------
for (i in 1:6) {
  jrwb_mmkin(f_dmta_mkin_const[2, i], caption_addition = "constant variance", label_addition = "cst")
  cat("  \\clearpage \n")
  jrwb_mmkin(f_dmta_mkin_tc[2, i], caption_addition = "two-component error", label_addition = "tc")
  cat("  \\clearpage \n")
}


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_dmta_nlme_3_obs, "DFOP-SFO3 fit with nlme to dimethenamid data, variance by variable", "lst:f_dmta_nlme_3_obs")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_dmta_nlme_3_tc, "DFOP-SFO3 fit with nlme to dimethenamid data, two-component error", "lst:f_dmta_nlme_3_tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_dmta_nlme_4_obs, "DFOP-SFO3+ fit with nlme to dimethenamid data, variance by variable", "lst:f_dmta_nlme_4_obs")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_dmta_nlme_4_tc, "DFOP-SFO3+ fit with nlme to dimethenamid data, two-component error", "lst:f_dmta_nlme_4_tc")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_dmta_saem_1_const, "SFO-SFO3 fit with saemix to dimethenamid data, constant variance", "lst:f_dmta_saem_1_const")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_dmta_saem_2_const, "SFO-SFO3+ fit with saemix to dimethenamid data, constant variance", "lst:f_dmta_saem_2_const")


## ----results = 'asis', cache = FALSE------------------------------------------
jrwb_mkin(f_dmta_saem_4_tc, "DFOP-SFO3+ fit with saemix to dimethenamid data, two-component error", "lst:f_dmta_saem_4_tc")

